@@ -31,19 +31,13 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)encrypt.c	8.2 (Berkeley) 5/30/95";
-#endif /* not lint */
-
-#ifndef lint
-static char orig_sccsid[] = "@(#)encrypt.c	5.2 (Berkeley) 3/22/91";
-#endif /* not lint */
+/* based on @(#)encrypt.c	8.1 (Berkeley) 6/4/93 */
 
 /*
  * Copyright (C) 1990 by the Massachusetts Institute of Technology
  *
- * Export of this software from the United States of America is assumed
- * to require a specific license from the United States Government.
+ * Export of this software from the United States of America may
+ * require a specific license from the United States Government.
  * It is the responsibility of any person or organization contemplating
  * export to obtain such a license before exporting.
  *
@@ -54,16 +48,17 @@ static char orig_sccsid[] = "@(#)encrypt.c	5.2 (Berkeley) 3/22/91";
  * this permission notice appear in supporting documentation, and that
  * the name of M.I.T. not be used in advertising or publicity pertaining
  * to distribution of the software without specific, written prior
- * permission.  M.I.T. makes no representations about the suitability of
+ * permission.  Furthermore if you modify this software you must label
+ * your software as modified software and not distribute it in such a
+ * fashion that it might be confused with the original M.I.T. software.
+ * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
  */
 
+#ifdef	ENCRYPTION
 
-#include <config.h>
-
-#if	defined(ENCRYPTION)
-
+#include <stdio.h>
 #define	ENCRYPT_NAMES
 #include <arpa/telnet.h>
 
@@ -73,18 +68,18 @@ static char orig_sccsid[] = "@(#)encrypt.c	5.2 (Berkeley) 3/22/91";
 #ifdef	__STDC__
 #include <stdlib.h>
 #endif
-#ifdef	NO_STRING_H
-#include <strings.h>
-#else
+#ifdef	HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
 
 /*
  * These functions pointers point to the current routines
  * for encrypting and decrypting data.
  */
-void	(*encrypt_output) P((unsigned char *, int));
-int	(*decrypt_input) P((int));
+void	(*encrypt_output) (unsigned char *, int);
+int	(*decrypt_input) (int);
 
 int encrypt_debug_mode = 0;
 static int decrypt_mode = 0;
@@ -111,7 +106,7 @@ static long remote_supports_encrypt = 0;
 static long remote_supports_decrypt = 0;
 
 static Encryptions encryptions[] = {
-#if	defined(DES_ENCRYPTION)
+#ifdef	DES_ENCRYPTION
     { "DES_CFB64",	ENCTYPE_DES_CFB64,
 			cfb64_encrypt,	
 			cfb64_decrypt,
@@ -132,7 +127,7 @@ static Encryptions encryptions[] = {
 			ofb64_session,
 			ofb64_keyid,
 			ofb64_printsub },
-#endif
+#endif	/* DES_ENCRYPTION */
     { 0, },
 };
 
@@ -219,7 +214,7 @@ encrypt_init(name, server)
 	str_send[str_suplen++] = SE;
 }
 
-	void
+static	void
 encrypt_list_types()
 {
 	Encryptions *ep = encryptions;
@@ -255,7 +250,7 @@ EncryptDisable(type, mode)
 	if (isprefix(type, "help") || isprefix(type, "?")) {
 		printf("Usage: encrypt disable <type> [input|output]\n");
 		encrypt_list_types();
-	} else if ((ep = (Encryptions *)genget(type, (char**)encryptions,
+	} else if ((ep = (Encryptions *)genget(type, (char **) encryptions,
 						sizeof(Encryptions))) == 0) {
 		printf("%s: invalid encryption type\n", type);
 	} else if (Ambiguous(ep)) {
@@ -290,7 +285,7 @@ EncryptType(type, mode)
 	if (isprefix(type, "help") || isprefix(type, "?")) {
 		printf("Usage: encrypt type <type> [input|output]\n");
 		encrypt_list_types();
-	} else if ((ep = (Encryptions *)genget(type, (char**)encryptions,
+	} else if ((ep = (Encryptions *)genget(type, (char **) encryptions,
 						sizeof(Encryptions))) == 0) {
 		printf("%s: invalid encryption type\n", type);
 	} else if (Ambiguous(ep)) {
@@ -341,7 +336,8 @@ EncryptStartInput()
 		encrypt_send_request_start();
 		return(1);
 	}
-	printf("No previous decryption mode, decryption not enabled\r\n");
+	if (!Server)
+	    printf("No previous decryption mode, decryption not enabled\r\n");
 	return(0);
 }
 
@@ -352,7 +348,8 @@ EncryptStartOutput()
 		encrypt_start_output(encrypt_mode);
 		return(1);
 	}
-	printf("No previous encryption mode, encryption not enabled\r\n");
+	if (!Server)
+	    printf("No previous encryption mode, encryption not enabled\r\n");
 	return(0);
 }
 
@@ -395,28 +392,17 @@ EncryptStopOutput()
 	void
 encrypt_display()
 {
-	printf("Autoencrypt for output is %s. Autodecrypt for input is %s.\r\n",
-	       autoencrypt?"on":"off", autodecrypt?"on":"off");
-
 	if (encrypt_output)
 		printf("Currently encrypting output with %s\r\n",
-		       ENCTYPE_NAME(encrypt_mode));
-	else
-		printf("Currently not encrypting output\r\n");
-	
+			ENCTYPE_NAME(encrypt_mode));
 	if (decrypt_input)
 		printf("Currently decrypting input with %s\r\n",
-		       ENCTYPE_NAME(decrypt_mode));
-	else
-		printf("Currently not decrypting input\r\n");
+			ENCTYPE_NAME(decrypt_mode));
 }
 
 	int
 EncryptStatus()
 {
-	printf("Autoencrypt for output is %s. Autodecrypt for input is %s.\r\n",
-	       autoencrypt?"on":"off", autodecrypt?"on":"off");
-
 	if (encrypt_output)
 		printf("Currently encrypting output with %s\r\n",
 			ENCTYPE_NAME(encrypt_mode));
@@ -424,9 +410,7 @@ EncryptStatus()
 		printf("Currently output is clear text.\r\n");
 		printf("Last encryption mode was %s\r\n",
 			ENCTYPE_NAME(encrypt_mode));
-	} else
-		printf("Currently not encrypting output\r\n");
-	
+	}
 	if (decrypt_input) {
 		printf("Currently decrypting input with %s\r\n",
 			ENCTYPE_NAME(decrypt_mode));
@@ -434,9 +418,7 @@ EncryptStatus()
 		printf("Currently input is clear text.\r\n");
 		printf("Last decryption mode was %s\r\n",
 			ENCTYPE_NAME(decrypt_mode));
-	} else
-		printf("Currently not decrypting input\r\n");
-
+	}
 	return 1;
 }
 
@@ -581,7 +563,8 @@ encrypt_is(data, cnt)
 	} else {
 		ret = (*ep->is)(data, cnt);
 		if (encrypt_debug_mode)
-			printf("(*ep->is)(%x, %d) returned %s(%d)\n", data, cnt,
+			printf("(*ep->is)(%lx, %d) returned %s(%d)\n", 
+			        (unsigned long) data, cnt,
 				(ret < 0) ? "FAIL " :
 				(ret == 0) ? "SUCCESS " : "MORE_TO_DO ", ret);
 	}
@@ -625,8 +608,8 @@ encrypt_reply(data, cnt)
 	} else {
 		ret = (*ep->reply)(data, cnt);
 		if (encrypt_debug_mode)
-			printf("(*ep->reply)(%x, %d) returned %s(%d)\n",
-				data, cnt,
+			printf("(*ep->reply)(%lx, %d) returned %s(%d)\n",
+				(unsigned long) data, cnt,
 				(ret < 0) ? "FAIL " :
 				(ret == 0) ? "SUCCESS " : "MORE_TO_DO ", ret);
 	}
@@ -663,7 +646,7 @@ encrypt_start(data, cnt)
 		return;
 	}
 
-	if (ep = finddecryption(decrypt_mode)) {
+	if ((ep = finddecryption(decrypt_mode))) {
 		decrypt_input = ep->input;
 		if (encrypt_verbose)
 			printf("[ Input is now decrypted with type %s ]\r\n",
@@ -694,6 +677,12 @@ encrypt_session_key(key, server)
 	while (ep->type) {
 		if (ep->session)
 			(*ep->session)(key, server);
+#ifdef notdef
+		if (!encrypt_output && autoencrypt && !server)
+			encrypt_start_output(ep->type);
+		if (!decrypt_input && autodecrypt && !server)
+			encrypt_send_request_start();
+#endif
 		++ep;
 	}
 }
@@ -741,27 +730,28 @@ encrypt_request_start(data, cnt)
 
 static unsigned char str_keyid[(MAXKEYLEN*2)+5] = { IAC, SB, TELOPT_ENCRYPT };
 
-encrypt_enc_keyid(keyid, len)
+static void encrypt_keyid (struct key_info *kp, unsigned char *, unsigned int);
+		
+void encrypt_enc_keyid(keyid, len)
 	unsigned char *keyid;
 	int len;
 {
-	encrypt_keyid(&ki[1], keyid, len);
+	encrypt_keyid(&ki[1], keyid, (unsigned) len);
 }
 
-encrypt_dec_keyid(keyid, len)
+void encrypt_dec_keyid(keyid, len)
 	unsigned char *keyid;
 	int len;
 {
-	encrypt_keyid(&ki[0], keyid, len);
+	encrypt_keyid(&ki[0], keyid, (unsigned) len);
 }
 
-encrypt_keyid(kp, keyid, len)
+static void encrypt_keyid(kp, keyid, len)
 	struct key_info *kp;
 	unsigned char *keyid;
-	int len;
+	unsigned int len;
 {
 	Encryptions *ep;
-	unsigned char *strp, *cp;
 	int dir = kp->dir;
 	register int ret = 0;
 
@@ -779,12 +769,13 @@ encrypt_keyid(kp, keyid, len)
 		if (ep->keyid)
 			(void)(*ep->keyid)(dir, kp->keyid, &kp->keylen);
 
-	} else if ((len != kp->keylen) || (memcmp(keyid,kp->keyid,len) != 0)) {
+	} else if ((len != kp->keylen) || 
+		   (memcmp(keyid, kp->keyid, len) != 0)) {
 		/*
 		 * Length or contents are different
 		 */
 		kp->keylen = len;
-		memcpy(kp->keyid,keyid, len);
+		memcpy(kp->keyid, keyid, len);
 		if (ep->keyid)
 			(void)(*ep->keyid)(dir, kp->keyid, &kp->keylen);
 	} else {
@@ -795,14 +786,14 @@ encrypt_keyid(kp, keyid, len)
 		return;
 	}
 
-	encrypt_send_keyid(dir, kp->keyid, kp->keylen, 0);
+	encrypt_send_keyid(dir, kp->keyid, (unsigned) kp->keylen, 0);
 }
 
 	void
 encrypt_send_keyid(dir, keyid, keylen, saveit)
 	int dir;
 	unsigned char *keyid;
-	int keylen;
+	unsigned int keylen;
 	int saveit;
 {
 	unsigned char *strp;
@@ -811,7 +802,7 @@ encrypt_send_keyid(dir, keyid, keylen, saveit)
 			? ENCRYPT_ENC_KEYID : ENCRYPT_DEC_KEYID;
 	if (saveit) {
 		struct key_info *kp = &ki[(dir == DIR_ENCRYPT) ? 0 : 1];
-		memcpy(kp->keyid,keyid, keylen);
+		memcpy(kp->keyid, keyid, keylen);
 		kp->keylen = keylen;
 	}
 
@@ -957,7 +948,6 @@ encrypt_send_request_end()
 	void
 encrypt_wait()
 {
-	register int encrypt, decrypt;
 	if (encrypt_debug_mode)
 		printf(">>>%s: in encrypt_wait\r\n", Name);
 	if (!havesessionkey || !(I_SUPPORT_ENCRYPT & remote_supports_decrypt))
@@ -967,6 +957,13 @@ encrypt_wait()
 			return;
 }
 
+int encrypt_is_encrypting()
+{
+    if (encrypt_output && decrypt_input)
+	return 1;
+    return 0;
+}
+
 	void
 encrypt_debug(mode)
 	int mode;
@@ -1012,4 +1009,6 @@ encrypt_printsub(data, cnt, buf, buflen)
 	else
 		encrypt_gen_printsub(data, cnt, buf, buflen);
 }
-#endif
+#else	/* ENCRYPTION */
+#include "misc-proto.h"
+#endif	/* ENCRYPTION */
