@@ -56,6 +56,9 @@
 
 #ifdef KRB5_KRB4_COMPAT
 
+#include "k5-int.h"
+#include "com_err.h"
+
 #include <des.h>
 #include <krb.h>
 #include <krb_db.h>
@@ -64,15 +67,15 @@
 
 static C_Block master_key;
 static Key_schedule master_key_schedule;
-static long master_key_version;
 
 static char *v4_mkeyfile = "/.k";
 
-#include "k5-int.h"
-#include "com_err.h"
 #include <kadm5/admin.h>
 #include <stdio.h>
-
+/* Define to make certain blocks private */
+#define V4_DECLARES_STATIC
+#include "kdb5_util.h"
+#include "kadm5/adb.h"                  /* osa_adb_create_policy_db */
 #include <netinet/in.h>			/* ntohl */
 
 #define PROGNAME argv[0]
@@ -101,20 +104,22 @@ static struct realm_info rblock = { /* XXX */
 
 static int verbose = 0;
 
+static int shortlife = 0;
+
 static krb5_error_code add_principal 
-	PROTOTYPE((krb5_context,
+		  (krb5_context,
 		   krb5_principal, 
 		   enum ap_op,
-		   struct realm_info *));
+		   struct realm_info *);
 
-static int v4init PROTOTYPE((char *, int, char *));
-static krb5_error_code enter_in_v5_db PROTOTYPE((krb5_context,
-						 char *, Principal *));
-static krb5_error_code process_v4_dump PROTOTYPE((krb5_context, char *,
-						  char *, long));
-static krb5_error_code v4_dump_find_default PROTOTYPE((krb5_context, char *,
-						       char *, long *));
-static krb5_error_code fixup_database PROTOTYPE((krb5_context, char *));
+static int v4init (char *, int, char *);
+static krb5_error_code enter_in_v5_db (krb5_context,
+				       char *, Principal *);
+static krb5_error_code process_v4_dump (krb5_context, char *,
+					char *, long);
+static krb5_error_code v4_dump_find_default (krb5_context, char *,
+					     char *, long *);
+static krb5_error_code fixup_database (krb5_context, char *);
 	
 static int create_local_tgt = 0;
 
@@ -149,8 +154,8 @@ static krb5_principal_data db_create_princ = {
 
 void
 load_v4db(argc, argv)
-int argc;
-char *argv[];
+    int argc;
+    char *argv[];
 {
     krb5_error_code retval;
     /* The kdb library will default to this, but it is convenient to
@@ -163,7 +168,7 @@ char *argv[];
     char *mkey_fullname;
     char *defrealm;
     int v4manual = 0;
-    int read_mkey = 0;
+    krb5_boolean read_mkey = 0;
     int tempdb = 0;
     char *tempdbname;
     krb5_context context;
@@ -201,6 +206,9 @@ char *argv[];
 	}
 	else if (!strcmp(argv[op_ind], "-n")) {
 	    v4manual++;
+	} 
+	else if (!strcmp(argv[op_ind], "-S")) {
+	    shortlife++;
 	}
 	else if (!strcmp(argv[op_ind], "-s")) {
 	    if ((argc - op_ind) >= 1) {
@@ -239,7 +247,7 @@ char *argv[];
 	return;
     }
 
-    if (!valid_enctype(master_keyblock.enctype)) {
+    if (!krb5_c_valid_enctype(master_keyblock.enctype)) {
 	com_err(PROGNAME, KRB5_PROG_KEYTYPE_NOSUPP,
 		"while setting up enctype %d", master_keyblock.enctype);
 	krb5_free_context(context);
@@ -258,7 +266,7 @@ char *argv[];
 	}
 	tempdbname = dbname;
     } else {
-	int dbnamelen = strlen(dbname);
+	size_t dbnamelen = strlen(dbname);
 	tempdbname = malloc(dbnamelen + 2);
 	if (tempdbname == 0) {
 	    com_err(PROGNAME, ENOMEM, "allocating temporary filename");
@@ -273,7 +281,8 @@ char *argv[];
 	
 
     if (!realm) {
-	if (retval = krb5_get_default_realm(context, &defrealm)) {
+	retval = krb5_get_default_realm(context, &defrealm);
+	if (retval) {
 	    com_err(PROGNAME, retval, "while retrieving default realm name");
 	    krb5_free_context(context);
 	    return;
@@ -282,9 +291,10 @@ char *argv[];
     }
 
     /* assemble & parse the master key name */
-
-    if (retval = krb5_db_setup_mkey_name(context, mkey_name, realm,
-					 &mkey_fullname, &master_princ)) {
+    
+    retval = krb5_db_setup_mkey_name(context, mkey_name, realm,
+				     &mkey_fullname, &master_princ);
+    if (retval) {
 	com_err(PROGNAME, retval, "while setting up master key name");
 	krb5_free_context(context);
 	return;
@@ -307,10 +317,12 @@ master key name '%s'\n",
 	fflush(stdout);
     }
 
-    if (retval = krb5_db_fetch_mkey(context, master_princ,
-				    master_keyblock.enctype,
-				    read_mkey, read_mkey, stash_file, 0, 
-				    &master_keyblock)) {
+
+    retval = krb5_db_fetch_mkey(context, master_princ,
+				master_keyblock.enctype,
+				read_mkey, read_mkey, stash_file, 0, 
+				&master_keyblock);
+    if (retval) {
 	com_err(PROGNAME, retval, "while reading master key");
 	krb5_free_context(context);
 	return;
@@ -321,18 +333,23 @@ master key name '%s'\n",
     seed.length = master_keyblock.length;
     seed.data = master_keyblock.contents;
 
-    if (retval = krb5_c_random_seed(context, &seed)) {
+    retval = krb5_c_random_seed(context, &seed);
+    if (retval) {
 	com_err(PROGNAME, retval, "while initializing random key generator");
 	krb5_free_context(context);
 	return;
     }
-    if (retval = krb5_db_create(context, tempdbname, crflags)) {
+
+    retval = krb5_db_create(context, tempdbname, crflags);
+    if (retval) {
 	com_err(PROGNAME, retval, "while creating %sdatabase '%s'",
 		tempdb ? "temporary " : "", tempdbname);
 	krb5_free_context(context);
 	return;
     }
-    if (retval = krb5_db_set_name(context, tempdbname)) {
+
+    retval = krb5_db_set_name(context, tempdbname);
+    if (retval) {
 	(void) krb5_db_destroy(context, tempdbname);
         com_err(PROGNAME, retval, "while setting active database to '%s'",
                 tempdbname);
@@ -353,7 +370,8 @@ master key name '%s'\n",
 	return;
     }
 
-    if (retval = add_principal(context, master_princ, MASTER_KEY, &rblock)) {
+    retval = add_principal(context, master_princ, MASTER_KEY, &rblock);
+    if (retval) {
 	(void) krb5_db_fini(context);
 	(void) krb5_db_destroy(context, tempdbname);
 	com_err(PROGNAME, retval, "while adding K/M to the database");
@@ -385,15 +403,16 @@ master key name '%s'\n",
     
     /* clean up; rename temporary database if there were no errors */
     if (retval == 0) {
-	if (retval = krb5_db_fini (context))
+	retval = krb5_db_fini (context);
+	if (retval)
 	    com_err(PROGNAME, retval, "while shutting down database");
 	else if (tempdb && (retval = krb5_db_rename(context, tempdbname,
-							dbname)))
+						    dbname)))
 	    com_err(PROGNAME, retval, "while renaming temporary database");
     } else {
 	(void) krb5_db_fini (context);
 	if (tempdb)
-		(void) krb5_db_destroy (context, tempdbname);
+	    (void) krb5_db_destroy (context, tempdbname);
     }
     memset((char *)master_keyblock.contents, 0, master_keyblock.length);
 
@@ -416,7 +435,8 @@ master key name '%s'\n",
     /*
      * Create the magic principals in the database.
      */
-    if (retval = kadm5_create_magic_princs(&newparams, context)) {
+    retval = kadm5_create_magic_princs(&newparams, context);
+    if (retval) {
 	com_err(PROGNAME, retval, "while creating KADM5 principals");
 	krb5_free_context(context);
 	return;
@@ -444,7 +464,7 @@ char *dumpfile;
 	}
     }
     if (!ok) {
-	des_read_password(master_key, "V4 Kerberos master key: ", 0);
+	des_read_password(&master_key, "V4 Kerberos master key", 0);
 	printf("\n");
     }
     key_sched(master_key, master_key_schedule);
@@ -504,28 +524,34 @@ Principal *princ;
 	return 0;
     }
     memset((char *) &entry, 0, sizeof(entry));
-    if (retval = krb5_425_conv_principal(context, princ->name, princ->instance,
-					 realm, &entry.princ))
+    retval = krb5_425_conv_principal(context, princ->name, princ->instance,
+				     realm, &entry.princ);
+    if (retval)
 	return retval;
     if (verbose) {
-	if (retval = krb5_unparse_name(context, entry.princ, &name))
+	retval = krb5_unparse_name(context, entry.princ, &name);
+	if (retval)
 	   name = strdup("<not unparsable name!>");
 	if (verbose)
 	    printf("\ntranslating %s...", name);
 	free(name);
     }
 
-    if (retval = krb5_build_principal(context, &mod_princ,
-				      strlen(realm),
-				      realm, princ->mod_name,
-				      princ->mod_instance[0] ? princ->mod_instance : 0,
-				      0)) {
+    retval = krb5_build_principal(context, &mod_princ,
+				  strlen(realm), realm, princ->mod_name,
+				  princ->mod_instance[0] ? 
+				  princ->mod_instance : NULL,
+				  NULL);
+    if (retval) {
 	krb5_free_principal(context, entry.princ);
 	return retval;
     }
     mod_time = princ->mod_date;
 
-    entry.max_life = princ->max_life * 60 * 5;
+    if (!shortlife)
+	entry.max_life = krb_life_to_time(0, princ->max_life);
+    else
+	entry.max_life = princ->max_life * 60 * 5;
     entry.max_renewable_life = rblock.max_rlife;
     entry.len = KRB5_KDB_V1_BASE_LENGTH;
     entry.expiration = princ->exp_date;
@@ -578,10 +604,11 @@ Principal *princ;
     if (!retval && !strcmp(princ->name, "krbtgt") &&
 	strcmp(princ->instance, realm) && princ->instance[0]) {
 	    krb5_free_principal(context, entry.princ);
-	    if (retval = krb5_build_principal(context, &entry.princ,
-					      strlen(princ->instance),
-					      princ->instance,
-					      "krbtgt", realm, 0))
+	    retval = krb5_build_principal(context, &entry.princ,
+					  strlen(princ->instance),
+					  princ->instance,
+					  "krbtgt", realm, NULL);
+	    if (retval)
 		    return retval;
 	    retval = krb5_db_put_principal(context, &entry, &nentries);
     }
@@ -604,23 +631,25 @@ struct realm_info *pblock;
     krb5_keyblock rkey;
     int nentries = 1;
     krb5_timestamp mod_time;
-    krb5_principal mod_princ;
 
     memset((char *) &entry, 0, sizeof(entry));
-    if (retval = krb5_copy_principal(context, princ, &entry.princ))
+    retval = krb5_copy_principal(context, princ, &entry.princ);
+    if (retval)
 	return(retval);
     entry.max_life = pblock->max_life;
     entry.max_renewable_life = pblock->max_rlife;
     entry.len = KRB5_KDB_V1_BASE_LENGTH;
     entry.expiration = pblock->expiration;
     
-    if ((retval = krb5_timeofday(context, &mod_time))) {
+    retval = krb5_timeofday(context, &mod_time);
+    if (retval) {
 	krb5_db_free_principal(context, &entry, 1);
 	return retval;
     }
     entry.attributes = pblock->flags;
 
-    if (retval = krb5_dbe_create_key_data(context, &entry)) {
+    retval = krb5_dbe_create_key_data(context, &entry);
+    if (retval) {
 	krb5_db_free_principal(context, &entry, 1);
 	return(retval);
     }
@@ -628,24 +657,26 @@ struct realm_info *pblock;
     switch (op) {
     case MASTER_KEY:
 	entry.attributes |= KRB5_KDB_DISALLOW_ALL_TIX;
-	if (retval = krb5_dbekd_encrypt_key_data(context, pblock->key,
-						 &master_keyblock,
-						 (krb5_keysalt *) NULL, 1,
-						 &entry.key_data[0])) {
+	retval = krb5_dbekd_encrypt_key_data(context, pblock->key,
+					     &master_keyblock,
+					     (krb5_keysalt *) NULL, 1,
+					     &entry.key_data[0]);
+	if (retval) {
 	    krb5_db_free_principal(context, &entry, 1);
 	    return retval;
 	}
 	break;
     case RANDOM_KEY:
-	if (retval = krb5_c_make_random_key(context, pblock->key->enctype,
-					    &rkey)) {
+	retval = krb5_c_make_random_key(context, pblock->key->enctype,
+					&rkey);
+	if (retval) {
 	    krb5_db_free_principal(context, &entry, 1);
 	    return retval;
 	}
-	if (retval = krb5_dbekd_encrypt_key_data(context, pblock->key,
-						 &rkey,
-						 (krb5_keysalt *) NULL, 1,
-						 &entry.key_data[0])) {
+	retval = krb5_dbekd_encrypt_key_data(context, pblock->key,
+					     &rkey, (krb5_keysalt *) NULL, 
+					     1, &entry.key_data[0]);
+	if (retval) {
 	    krb5_db_free_principal(context, &entry, 1);
 	    return(retval);
 	}
@@ -795,7 +826,6 @@ long default_exp_time;
     char    exp_date_str[50];
     char    mod_date_str[50];
     int     temp1, temp2, temp3;
-    long time_explode();
 
     input_file = fopen(dumpfile, "r");
     if (!input_file)
@@ -806,7 +836,7 @@ long default_exp_time;
 
 	memset((char *)&aprinc, 0, sizeof(aprinc));
 	nread = fscanf(input_file,
-		       "%s %s %d %d %d %hd %x %x %s %s %s %s\n",
+		       "%s %s %d %d %d %hd %lx %lx %s %s %s %s\n",
 		       aprinc.name,
 		       aprinc.instance,
 		       &temp1,
@@ -838,7 +868,8 @@ long default_exp_time;
 	    aprinc.mod_name[0] = '\0';
 	if (aprinc.mod_instance[0] == '*')
 	    aprinc.mod_instance[0] = '\0';
-	if (retval = enter_in_v5_db(context, realm, &aprinc))
+	retval = enter_in_v5_db(context, realm, &aprinc);
+	if (retval)
 	    break;
     }
     (void) fclose(input_file);
@@ -858,7 +889,6 @@ long *exptime;
     char    exp_date_str[50];
     char    mod_date_str[50];
     int     temp1, temp2, temp3;
-    long time_explode();
     long foundtime, guess1, guess2;
 
     /* kdb_init is usually the only thing to touch the time in the 
@@ -893,7 +923,7 @@ long *exptime;
 
 	memset((char *)&aprinc, 0, sizeof(aprinc));
 	nread = fscanf(input_file,
-		       "%s %s %d %d %d %hd %x %x %s %s %s %s\n",
+		       "%s %s %d %d %d %hd %lx %lx %s %s %s %s\n",
 		       aprinc.name,
 		       aprinc.instance,
 		       &temp1,
@@ -922,7 +952,7 @@ long *exptime;
 		} else if (foundtime == guess2) {
 		    printf("Cygnus CNS post 96q1 value (2009)");
 		} else {
-		    printf("non-default start time (%d,%s)",
+		    printf("non-default start time (%ld,%s)",
 			   foundtime, exp_date_str);
 		}
 	    }
