@@ -8,7 +8,6 @@
 #define USE_KADM5_API_VERSION 1
 #include <kadm5/admin.h>
 #include <com_err.h>
-#include <k5-int.h>
 #include <errno.h>
 #include <stdlib.h>
 #include "tcl_kadm5.h"
@@ -470,7 +469,7 @@ static int parse_flags(Tcl_Interp *interp, Tcl_HashTable *table,
 	  *flags |= *(krb5_flags *) Tcl_GetHashValue(entry);
      }
   
-     free(argv);
+     Tcl_Free(argv);
      return(retcode);
 }
 
@@ -781,7 +780,7 @@ static int parse_principal_ent(Tcl_Interp *interp, char *list,
      }
 
 finished:
-     free(argv);
+     Tcl_Free(argv);
      *out_princ = princ;
      return retcode;
 }
@@ -930,7 +929,7 @@ static int parse_policy_ent(Tcl_Interp *interp, char *list,
      policy->policy_refcnt = tmp;
 
 finished:
-     free(argv);
+     Tcl_Free(argv);
      *out_policy = policy;
      return retcode;
 }
