@@ -512,7 +512,7 @@ tgt_again:
 			      enc_tkt_reply.transited.tr_contents.data,
 			      error_message (errcode));
     } else
-	krb5_klog_syslog (LOG_ERR, "not checking transit path");
+	krb5_klog_syslog (LOG_INFO, "not checking transit path");
     if (reject_bad_transit
 	&& !isflagset (enc_tkt_reply.flags, TKT_FLG_TRANSIT_POLICY_CHECKED)) {
 	errcode = KRB5KDC_ERR_POLICY;
@@ -722,7 +722,11 @@ prepare_error_tgs (krb5_kdc_req *request, krb5_ticket *ticket, int error,
 
     retval = krb5_mk_error(kdc_context, &errpkt, scratch);
     free(errpkt.text.data);
-    *response = scratch;
+    if (retval)
+	free(scratch);
+    else
+	*response = scratch;
+
     return retval;
 }
 
