@@ -126,7 +126,7 @@ static const char *paddr (struct sockaddr *sa)
 		    NI_NUMERICHOST|NI_NUMERICSERV))
 	strcpy(buf, "<unprintable>");
     else {
-	int len = sizeof(buf) - strlen(buf);
+	unsigned int len = sizeof(buf) - strlen(buf);
 	char *p = buf + strlen(buf);
 	if (len > 2+strlen(portbuf)) {
 	    *p++ = '.';
@@ -139,10 +139,12 @@ static const char *paddr (struct sockaddr *sa)
 
 /* KDC data.  */
 
+enum kdc_conn_type { CONN_UDP, CONN_TCP_LISTENER, CONN_TCP };
+
 /* Per-connection info.  */
 struct connection {
     int fd;
-    enum { CONN_UDP, CONN_TCP_LISTENER, CONN_TCP } type;
+    enum kdc_conn_type type;
     void (*service)(struct connection *, const char *, int);
     union {
 	/* Type-specific information.  */
@@ -262,7 +264,7 @@ static krb5_error_code add_tcp_port(int port)
 #define USE_TYPE SOCK_DGRAM
 #define USE_PROTO 0
 #define SOCKET_ERRNO errno
-#include "foreachaddr.c"
+#include "foreachaddr.h"
 
 struct socksetup {
     const char *prog;
@@ -270,7 +272,7 @@ struct socksetup {
 };
 
 static struct connection *
-add_fd (struct socksetup *data, int sock, int conntype,
+add_fd (struct socksetup *data, int sock, enum kdc_conn_type conntype,
 	void (*service)(struct connection *, const char *, int))
 {
     struct connection *newconn;
@@ -359,6 +361,21 @@ setup_a_tcp_listener(struct socksetup *data, struct sockaddr *addr)
 		paddr(addr));
 	return -1;
     }
+    if (setreuseaddr(sock, 1) < 0)
+	com_err(data->prog, errno,
+		"Cannot enable SO_REUSEADDR on fd %d", sock);
+#ifdef KRB5_USE_INET6
+    if (addr->sa_family == AF_INET6) {
+#ifdef IPV6_V6ONLY
+	if (setv6only(sock, 1))
+	    com_err(data->prog, errno, "setsockopt(IPV6_V6ONLY,1) failed");
+	else
+	    com_err(data->prog, 0, "setsockopt(IPV6_V6ONLY,1) worked");
+#else
+	krb5_klog_syslog(LOG_INFO, "no IPV6_V6ONLY socket option support");
+#endif /* IPV6_V6ONLY */
+    }
+#endif /* KRB5_USE_INET6 */
     if (bind(sock, addr, socklen(addr)) == -1) {
 	com_err(data->prog, errno,
 		"Cannot bind TCP server socket on %s", paddr(addr));
@@ -432,18 +449,6 @@ setup_tcp_listener_ports(struct socksetup *data)
 	    s6 = setup_a_tcp_listener(data, (struct sockaddr *)&sin6);
 	    if (s6 < 0)
 		return -1;
-#ifdef IPV6_V6ONLY
-	    if (setv6only(s6, 0))
-		com_err(data->prog, errno, "setsockopt(IPV6_V6ONLY,0) failed");
-#endif
-
-	    if (setreuseaddr(s6, 0) < 0) {
-		com_err(data->prog, errno,
-			"disabling SO_REUSEADDR on IPv6 TCP socket for port %d",
-			port);
-		close(s6);
-		return -1;
-	    }
 
 	    s4 = setup_a_tcp_listener(data, (struct sockaddr *)&sin4);
 #endif /* KRB5_USE_INET6 */
@@ -515,6 +520,10 @@ setup_udp_port(void *P_data, struct sockaddr *addr)
 #ifdef AF_LINK /* some BSD systems, AIX */
     case AF_LINK:
 	return 0;
+#endif
+#ifdef AF_DLI /* Direct Link Interface - DEC Ultrix/OSF1 link layer? */
+    case AF_DLI:
+	return 0;
 #endif
     default:
 	krb5_klog_syslog (LOG_INFO,
@@ -713,6 +722,7 @@ static void process_packet(struct connection *conn, const char *prog,
     char pktbuf[MAX_DGRAM_SIZE];
     int port_fd = conn->fd;
 
+    response = NULL;
     saddr_len = sizeof(saddr);
     cc = recvfrom(port_fd, pktbuf, sizeof(pktbuf), 0,
 		  (struct sockaddr *)&saddr, &saddr_len);
@@ -804,6 +814,10 @@ static void accept_tcp_connection(struct connection *conn, const char *prog,
 	    strcpy(p, tmpbuf);
 	}
     }
+#if 0
+    krb5_klog_syslog(LOG_INFO, "accepted TCP connection on socket %d from %s",
+		     s, newconn->u.tcp.addrbuf);
+#endif
 
     newconn->u.tcp.addr_s = addr_s;
     newconn->u.tcp.addrlen = addrlen;
@@ -884,12 +898,10 @@ process_tcp_connection(struct connection *conn, const char *prog, int selflags)
     if (selflags & SSF_WRITE) {
 	ssize_t nwrote;
 	SOCKET_WRITEV_TEMP tmp;
-	krb5_error_code e;
 
 	nwrote = SOCKET_WRITEV(conn->fd, conn->u.tcp.sgp, conn->u.tcp.sgnum,
 			       tmp);
 	if (nwrote < 0) {
-	    e = SOCKET_ERRNO;
 	    goto kill_tcp_connection;
 	}
 	if (nwrote == 0)
@@ -1058,6 +1070,11 @@ closedown_network(const char *prog)
 	if (conn->fd >= 0)
 	    (void) close(conn->fd);
 	DEL (connections, i);
+	/* There may also be per-connection data in the tcp structure
+	   (tcp.buffer, tcp.response) that we're not freeing here.
+	   That should only happen if we quit with a connection in
+	   progress.  */
+	free(conn);
     }
     FREE_SET_DATA(connections);
     FREE_SET_DATA(udp_port_data);
