@@ -41,7 +41,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include <krb5.h>
+#include <k5-int.h>
 #include <kadm5/admin.h>
 #include <krb5/adm_proto.h>
 #include <com_err.h>
@@ -329,7 +329,8 @@ krb5_error_code do_connection(s, context)
      krb5_ticket *v5tkt = 0;
      krb5_data msgdata, tktdata;
      char msgbuf[MSGSIZE], tktbuf[TKT_BUFSIZ], *p;
-     int ret, saddrlen;
+     int ret;
+     socklen_t saddrlen;
      krb5_int32 n; /* Must be 4 bytes */
      krb5_kvno v4kvno;
 
@@ -582,8 +583,10 @@ ret =  KRB5KDC_ERR_POLICY ;
 	  printf("v4 credentials encoded\n");
 
  error:
-     if (v5tkt->enc_part2)
+     if (v5tkt->enc_part2) {
 	 krb5_free_enc_tkt_part(context, v5tkt->enc_part2);
+	 v5tkt->enc_part2 = NULL;
+     }
 
      if(v5_service_key.contents)
        krb5_free_keyblock_contents(context, &v5_service_key);
