@@ -28,11 +28,8 @@
 #include "etypes.h"
 
 krb5_error_code KRB5_CALLCONV
-krb5_c_encrypt_length(context, enctype, inputlen, length)
-     krb5_context context;
-     krb5_enctype enctype;
-     size_t inputlen;
-     size_t *length;
+krb5_c_encrypt_length(krb5_context context, krb5_enctype enctype,
+		      size_t inputlen, size_t *length)
 {
     int i;
 
