@@ -26,5 +26,5 @@
 
 #include "k5-int.h"
 
-extern struct krb5_keytypes krb5_enctypes_list[];
-extern int krb5_enctypes_length;
+extern const struct krb5_keytypes krb5_enctypes_list[];
+extern const int krb5_enctypes_length;
