@@ -107,8 +107,7 @@ static const unsigned char PADDING[64] = {
    mdContext. All fields are set to zero.
  */
 void 
-krb5_MD5Init (mdContext)
-krb5_MD5_CTX *mdContext;
+krb5_MD5Init (krb5_MD5_CTX *mdContext)
 {
   mdContext->i[0] = mdContext->i[1] = (krb5_ui_4)0;
 
@@ -125,10 +124,7 @@ krb5_MD5_CTX *mdContext;
    in the message whose digest is being computed.
  */
 void
-krb5_MD5Update (mdContext, inBuf, inLen)
-krb5_MD5_CTX *mdContext;
-const unsigned char *inBuf;
-unsigned int inLen;
+krb5_MD5Update (krb5_MD5_CTX *mdContext, const unsigned char *inBuf, unsigned int inLen)
 {
   krb5_ui_4 in[16];
   int mdi;
@@ -164,8 +160,7 @@ unsigned int inLen;
    ends with the desired message digest in mdContext->digest[0...15].
  */
 void
-krb5_MD5Final (mdContext)
-krb5_MD5_CTX *mdContext;
+krb5_MD5Final (krb5_MD5_CTX *mdContext)
 {
   krb5_ui_4 in[16];
   int mdi;
@@ -205,9 +200,7 @@ krb5_MD5_CTX *mdContext;
 
 /* Basic MD5 step. Transforms buf based on in.
  */
-static void Transform (buf, in)
-krb5_ui_4 *buf;
-krb5_ui_4 *in;
+static void Transform (krb5_ui_4 *buf, krb5_ui_4 *in)
 {
   register krb5_ui_4 a = buf[0], b = buf[1], c = buf[2], d = buf[3];
 
