@@ -56,7 +56,7 @@
 #endif
 
 /*
- * $Id: add_cred.c,v 1.8 2003/03/04 20:45:32 hartmans Exp $
+ * $Id: add_cred.c 16630 2004-07-29 02:26:43Z raeburn $
  */
 
 /* V2 interface */
@@ -113,12 +113,21 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 	return(GSS_S_DUPLICATE_ELEMENT);
     }
 
-    /* verify the credential */
-    if (GSS_ERROR(major_status =
-		  krb5_gss_validate_cred(minor_status, input_cred_handle)))
-	return(major_status);
+    code = krb5_init_context(&context);
+    if (code) {
+	*minor_status = code;
+	return GSS_S_FAILURE;
+    }
+
+    major_status = krb5_gss_validate_cred_1(minor_status, input_cred_handle,
+					    context);
+    if (GSS_ERROR(major_status)) {
+	krb5_free_context(context);
+	return major_status;
+    }
 
     cred = (krb5_gss_cred_id_t) input_cred_handle;
+    k5_mutex_assert_locked(&cred->lock);
 
     /* check if the cred_usage is equal or "less" than the passed-in cred
        if copying */
@@ -127,6 +136,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 	  ((cred->usage == GSS_C_BOTH) &&
 	   (output_cred_handle != NULL)))) {
       *minor_status = (OM_uint32) G_BAD_USAGE;
+      krb5_free_context(context);
       return(GSS_S_FAILURE);
     }
 
@@ -135,11 +145,14 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
     if ((g_OID_equal(desired_mech, gss_mech_krb5_old) && cred->prerfc_mech) ||
 	(g_OID_equal(desired_mech, gss_mech_krb5) && cred->rfc_mech)) {
 	*minor_status = 0;
+	krb5_free_context(context);
 	return(GSS_S_DUPLICATE_ELEMENT);
     }
 
-    if (GSS_ERROR(kg_get_context(minor_status, &context)))
-	return(GSS_S_FAILURE);
+    if (GSS_ERROR(kg_sync_ccache_name(context, minor_status))) {
+	krb5_free_context(context);
+	return GSS_S_FAILURE;
+    }
 
     /* verify the desired_name */
 
@@ -147,6 +160,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
     if ((desired_name != (gss_name_t) NULL) &&
 	(! kg_validate_name(desired_name))) {
 	*minor_status = (OM_uint32) G_VALIDATE_FAILED;
+	krb5_free_context(context);
 	return(GSS_S_CALL_BAD_STRUCTURE|GSS_S_BAD_NAME);
     }
 
@@ -156,6 +170,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 	!krb5_principal_compare(context, (krb5_principal) desired_name,
 				cred->princ)) {
 	*minor_status = 0;
+	krb5_free_context(context);
 	return(GSS_S_BAD_NAME);
     }
 
@@ -172,6 +187,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 	     (krb5_gss_cred_id_t) xmalloc(sizeof(krb5_gss_cred_id_rec)))
 	    == NULL) {
 	    *minor_status = ENOMEM;
+	    krb5_free_context(context);
 	    return(GSS_S_FAILURE);
 	}
 	memset(new_cred, 0, sizeof(krb5_gss_cred_id_rec));
@@ -187,6 +203,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 	    xfree(new_cred);
 
 	    *minor_status = code;
+	    krb5_free_context(context);
 	    return(GSS_S_FAILURE);
 	}
 	    
@@ -198,6 +215,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 		xfree(new_cred);
 
 		*minor_status = ENOMEM;
+		krb5_free_context(context);
 		return(GSS_S_FAILURE);
 	    }
 
@@ -214,6 +232,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 		xfree(new_cred);
 
 		*minor_status = code;
+		krb5_free_context(context);
 		return(GSS_S_FAILURE);
 	    }
 
@@ -224,6 +243,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 		xfree(new_cred);
 
 		*minor_status = code;
+		krb5_free_context(context);
 		return(GSS_S_FAILURE);
 	    }
 	} else {
@@ -241,6 +261,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 		    krb5_free_principal(context, new_cred->princ);
 		xfree(new_cred);
 
+		krb5_free_context(context);
 		*minor_status = code;
 		return(GSS_S_FAILURE);
 	    }
@@ -261,6 +282,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 		krb5_free_principal(context, new_cred->princ);
 		xfree(new_cred);
 
+		krb5_free_context(context);
 		*minor_status = ENOMEM;
 		return(GSS_S_FAILURE);
 	    }
@@ -279,6 +301,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 		if (new_cred->princ)
 		    krb5_free_principal(context, new_cred->princ);
 		xfree(new_cred);
+		krb5_free_context(context);
 
 		*minor_status = code;
 		return(GSS_S_FAILURE);
@@ -299,6 +322,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 	    if (new_cred->princ)
 	    krb5_free_principal(context, new_cred->princ);
 	    xfree(new_cred);
+	    krb5_free_context(context);
 
 	    *minor_status = (OM_uint32) G_VALIDATE_FAILED;
 	    return(GSS_S_FAILURE);
@@ -325,6 +349,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
 	
 	if (output_cred_handle)
 	    (void) krb5_gss_release_cred(&dummy, (gss_cred_id_t *) &cred);
+	krb5_free_context(context);
 
 	return(major_status);
     }
@@ -337,6 +362,7 @@ krb5_gss_add_cred(minor_status, input_cred_handle,
     if (output_cred_handle)
 	*output_cred_handle = cred;
 
+    krb5_free_context(context);
     *minor_status = 0;
     return(GSS_S_COMPLETE);
 }
