@@ -47,7 +47,7 @@
 #ifndef _GSSAPIP_KRB5_H_
 #define _GSSAPIP_KRB5_H_
 
-#include <krb5.h>
+#include <k5-int.h>
 
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
@@ -141,6 +141,9 @@ enum qop {
 typedef krb5_principal krb5_gss_name_t;
 
 typedef struct _krb5_gss_cred_id_rec {
+   /* protect against simultaneous accesses */
+   k5_mutex_t lock;
+
    /* name/type of credential */
    gss_cred_usage_t usage;
    krb5_principal princ;	/* this is not interned as a gss_name_t */
@@ -154,6 +157,7 @@ typedef struct _krb5_gss_cred_id_rec {
    /* ccache (init) data */
    krb5_ccache ccache;
    krb5_timestamp tgt_expire;
+   krb5_enctype *req_enctypes;	/* limit negotiated enctypes to this list */
 } krb5_gss_cred_id_rec, *krb5_gss_cred_id_t; 
 
 typedef struct _krb5_gss_ctx_id_rec {
@@ -162,9 +166,6 @@ typedef struct _krb5_gss_ctx_id_rec {
    unsigned int big_endian : 1;
    unsigned int have_acceptor_subkey : 1;
    unsigned int seed_init : 1;	/* XXX tested but never actually set */
-#ifdef CFX_EXERCISE
-   unsigned int testing_unknown_tokid : 1; /* for testing only */
-#endif
    OM_uint32 gss_flags;
    unsigned char seed[16];
    krb5_principal here;
@@ -183,6 +184,7 @@ typedef struct _krb5_gss_ctx_id_rec {
    gssint_uint64 seq_send;
    gssint_uint64 seq_recv;
    void *seqstate;
+   krb5_context k5_context;
    krb5_auth_context auth_context;
    gss_OID_desc *mech_used;
     /* Protocol spec revision
@@ -193,26 +195,29 @@ typedef struct _krb5_gss_ctx_id_rec {
    krb5_cksumtype cksumtype;	/* for "main" subkey */
    krb5_keyblock *acceptor_subkey; /* CFX only */
    krb5_cksumtype acceptor_subkey_cksumtype;
-#ifdef CFX_EXERCISE
-    gss_buffer_desc init_token;
-#endif
+   int cred_rcache;		/* did we get rcache from creds? */
 } krb5_gss_ctx_id_rec, *krb5_gss_ctx_id_t;
 
-extern void *kg_vdb;
+extern g_set kg_vdb;
+
+extern k5_mutex_t gssint_krb5_keytab_lock;
 
 /* helper macros */
 
 #define kg_save_name(name)		g_save_name(&kg_vdb,name)
 #define kg_save_cred_id(cred)		g_save_cred_id(&kg_vdb,cred)
 #define kg_save_ctx_id(ctx)		g_save_ctx_id(&kg_vdb,ctx)
+#define kg_save_lucidctx_id(lctx)	g_save_lucidctx_id(&kg_vdb,lctx)
 
 #define kg_validate_name(name)		g_validate_name(&kg_vdb,name)
 #define kg_validate_cred_id(cred)	g_validate_cred_id(&kg_vdb,cred)
 #define kg_validate_ctx_id(ctx)		g_validate_ctx_id(&kg_vdb,ctx)
+#define kg_validate_lucidctx_id(lctx)	g_validate_lucidctx_id(&kg_vdb,lctx)
 
 #define kg_delete_name(name)		g_delete_name(&kg_vdb,name)
 #define kg_delete_cred_id(cred)		g_delete_cred_id(&kg_vdb,cred)
 #define kg_delete_ctx_id(ctx)		g_delete_ctx_id(&kg_vdb,ctx)
+#define kg_delete_lucidctx_id(lctx)	g_delete_lucidctx_id(&kg_vdb,lctx)
 
 /** helper functions **/
 
@@ -247,7 +252,7 @@ krb5_error_code kg_make_confounder (krb5_context context,
 krb5_error_code kg_encrypt (krb5_context context, 
 				      krb5_keyblock *key, int usage,
 				      krb5_pointer iv,
-				      krb5_pointer in,
+				      krb5_const_pointer in,
 				      krb5_pointer out,
 				      unsigned int length);
 krb5_error_code
@@ -259,12 +264,11 @@ kg_arcfour_docrypt (const krb5_keyblock *longterm_key , int ms_usage,
 krb5_error_code kg_decrypt (krb5_context context,
 				      krb5_keyblock *key,  int usage,
 				      krb5_pointer iv,
-				      krb5_pointer in,
+				      krb5_const_pointer in,
 				      krb5_pointer out,
 				      unsigned int length);
 
-OM_uint32 kg_seal (krb5_context context,
-		  OM_uint32 *minor_status,
+OM_uint32 kg_seal (OM_uint32 *minor_status,
 		  gss_ctx_id_t context_handle,
 		  int conf_req_flag,
 		  int qop_req,
@@ -273,8 +277,7 @@ OM_uint32 kg_seal (krb5_context context,
 		  gss_buffer_t output_message_buffer,
 		  int toktype);
 
-OM_uint32 kg_unseal (krb5_context context,
-		    OM_uint32 *minor_status,
+OM_uint32 kg_unseal (OM_uint32 *minor_status,
 		    gss_ctx_id_t context_handle,
 		    gss_buffer_t input_token_buffer,
 		    gss_buffer_t message_buffer,
@@ -282,8 +285,7 @@ OM_uint32 kg_unseal (krb5_context context,
 		    int *qop_state,
 		    int toktype);
 
-OM_uint32 kg_seal_size (krb5_context context,
-				  OM_uint32 *minor_status,
+OM_uint32 kg_seal_size (OM_uint32 *minor_status,
 				  gss_ctx_id_t context_handle,
 				  int conf_req_flag,
 				  gss_qop_t qop_req,
@@ -304,10 +306,7 @@ krb5_error_code kg_ctx_internalize (krb5_context kcontext,
 					      krb5_octet **buffer,
 					      size_t *lenremain);
 
-OM_uint32 kg_get_context (OM_uint32 *minor_status,
-				    krb5_context *context);
-
-OM_uint32 kg_sync_ccache_name (OM_uint32 *minor_status);
+OM_uint32 kg_sync_ccache_name (krb5_context context, OM_uint32 *minor_status);
 
 OM_uint32 kg_get_ccache_name (OM_uint32 *minor_status, 
                               const char **out_name);
@@ -574,6 +573,8 @@ OM_uint32 krb5_gss_import_sec_context
 	    gss_ctx_id_t *		/* context_handle */
 	    );
 
+krb5_error_code krb5_gss_ser_init(krb5_context);
+
 OM_uint32 krb5_gss_release_oid
 (OM_uint32 *,		/* minor_status */
 	    gss_OID *			/* oid */
@@ -609,9 +610,12 @@ OM_uint32 krb5_gss_validate_cred
 	    gss_cred_id_t		/* cred */
          );
 
-gss_OID krb5_gss_convert_static_mech_oid
-(gss_OID oid
-	 );
+OM_uint32
+krb5_gss_validate_cred_1(OM_uint32 * /* minor_status */,
+			 gss_cred_id_t /* cred_handle */,
+			 krb5_context /* context */);
+
+gss_OID krb5_gss_convert_static_mech_oid(gss_OID oid);
 	
 krb5_error_code gss_krb5int_make_seal_token_v3(krb5_context,
 					       krb5_gss_ctx_id_rec *,
