@@ -1,4 +1,5 @@
 /*
+ * Copyright 2001 by the Massachusetts Institute of Technology.
  * Copyright 1993 by OpenVision Technologies, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software
@@ -47,16 +48,15 @@
  */
 
 #include "gssapiP_krb5.h"
+#ifdef HAVE_MEMORY_H
 #include <memory.h>
-
-/*
- * $Id: k5unseal.c,v 1.19.6.2 2000/05/31 17:17:38 raeburn Exp $
- */
+#endif
+#include <assert.h>
 
 /* message_buffer is an input if SIGN, output if SEAL, and ignored if DEL_CTX
    conf_state is only valid if SEAL. */
 
-OM_uint32
+static OM_uint32
 kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 	     conf_state, qop_state, toktype)
     krb5_context context;
@@ -70,7 +70,6 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
     int toktype;
 {
     krb5_error_code code;
-    int tmsglen;
     int conflen = 0;
     int signalg;
     int sealalg;
@@ -84,9 +83,10 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
     int cksum_len = 0;
     int plainlen;
     int direction;
-    krb5_int32 seqnum;
+    krb5_ui_4 seqnum;
     OM_uint32 retval;
     size_t sumlen;
+    krb5_keyusage sign_usage = KG_USAGE_SIGN;
 
     if (toktype == KG_TOK_SEAL_MSG) {
 	message_buffer->length = 0;
@@ -127,7 +127,9 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
     if ((ctx->sealalg == SEAL_ALG_NONE && signalg > 1) ||
 	(ctx->sealalg == SEAL_ALG_1 && signalg != SGN_ALG_3) ||
 	(ctx->sealalg == SEAL_ALG_DES3KD &&
-	 signalg != SGN_ALG_HMAC_SHA1_DES3_KD)) {
+	 signalg != SGN_ALG_HMAC_SHA1_DES3_KD)||
+	(ctx->sealalg == SEAL_ALG_MICROSOFT_RC4 &&
+	signalg != SGN_ALG_HMAC_MD5)) {
 	*minor_status = 0;
 	return GSS_S_DEFECTIVE_TOKEN;
     }
@@ -135,8 +137,11 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
     switch (signalg) {
     case SGN_ALG_DES_MAC_MD5:
     case SGN_ALG_MD2_5:
+    case SGN_ALG_HMAC_MD5:
 	cksum_len = 8;
-	break;
+	if (toktype != KG_TOK_SEAL_MSG)
+	  sign_usage = 15;
+	    break;
     case SGN_ALG_3:
 	cksum_len = 16;
 	break;
@@ -148,23 +153,53 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 	return GSS_S_DEFECTIVE_TOKEN;
     }
 
-    if (toktype == KG_TOK_SEAL_MSG)
-	tmsglen = bodysize-(14+cksum_len);
-
     /* get the token parameters */
 
+    if ((code = kg_get_seq_num(context, ctx->seq, ptr+14, ptr+6, &direction,
+			       &seqnum))) {
+	*minor_status = code;
+	return(GSS_S_BAD_SIG);
+    }
+
     /* decode the message, if SEAL */
 
     if (toktype == KG_TOK_SEAL_MSG) {
+	int tmsglen = bodysize-(14+cksum_len);
 	if (sealalg != 0xffff) {
 	    if ((plain = (unsigned char *) xmalloc(tmsglen)) == NULL) {
 		*minor_status = ENOMEM;
 		return(GSS_S_FAILURE);
 	    }
+	    if (ctx->enc->enctype == ENCTYPE_ARCFOUR_HMAC) {
+	      unsigned char bigend_seqnum[4];
+	      krb5_keyblock *enc_key;
+	      int i;
+	      bigend_seqnum[0] = (seqnum>>24) & 0xff;
+	      bigend_seqnum[1] = (seqnum>>16) & 0xff;
+	      bigend_seqnum[2] = (seqnum>>8) & 0xff;
+	      bigend_seqnum[3] = seqnum & 0xff;
+	      code = krb5_copy_keyblock (context, ctx->enc, &enc_key);
+	      if (code)
+		{
+		  xfree(plain);
+		  *minor_status = code;
+		  return(GSS_S_FAILURE);
+		}
 
-	    if ((code = kg_decrypt(context, ctx->enc, KG_USAGE_SEAL, NULL,
-				   ptr+14+cksum_len, plain, tmsglen))) {
-		xfree(plain);
+	      assert (enc_key->length == 16);
+	      for (i = 0; i <= 15; i++)
+		((char *) enc_key->contents)[i] ^=0xf0;
+	      code = kg_arcfour_docrypt (enc_key, 0,
+					 &bigend_seqnum[0], 4, 
+					 ptr+14+cksum_len, tmsglen,
+					 plain);
+	      krb5_free_keyblock (context, enc_key);
+	    } else {
+	      code = kg_decrypt(context, ctx->enc, KG_USAGE_SEAL, NULL,
+				ptr+14+cksum_len, plain, tmsglen);
+		}
+	    if (code) {
+	      		xfree(plain);
 		*minor_status = code;
 		return(GSS_S_FAILURE);
 	    }
@@ -189,6 +224,8 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 		return(GSS_S_FAILURE);
 	    }
 	    memcpy(token.value, plain+conflen, token.length);
+	} else {
+	    token.value = NULL;
 	}
     } else if (toktype == KG_TOK_SIGN_MSG) {
 	token = *message_buffer;
@@ -207,11 +244,13 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
     switch (signalg) {
     case SGN_ALG_DES_MAC_MD5:
     case SGN_ALG_MD2_5:
-    case SGN_ALG_HMAC_MD5:
     case SGN_ALG_DES_MAC:
     case SGN_ALG_3:
 	md5cksum.checksum_type = CKSUMTYPE_RSA_MD5;
 	break;
+    case SGN_ALG_HMAC_MD5:
+      md5cksum.checksum_type = CKSUMTYPE_HMAC_MD5_ARCFOUR;
+      break;
     case SGN_ALG_HMAC_SHA1_DES3_KD:
 	md5cksum.checksum_type = CKSUMTYPE_HMAC_SHA1_DES3;
 	break;
@@ -219,7 +258,8 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 	abort ();
     }
 
-    if (code = krb5_c_checksum_length(context, md5cksum.checksum_type, &sumlen))
+    code = krb5_c_checksum_length(context, md5cksum.checksum_type, &sumlen);
+    if (code)
 	return(code);
     md5cksum.length = sumlen;
 
@@ -250,7 +290,7 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 	plaind.length = 8 + (ctx->big_endian ? token.length : plainlen);
 	plaind.data = data_ptr;
 	code = krb5_c_make_checksum(context, md5cksum.checksum_type,
-				    ctx->seq, KG_USAGE_SIGN,
+				    ctx->seq, sign_usage,
 				    &plaind, &md5cksum);
 	xfree(data_ptr);
 
@@ -265,7 +305,7 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 			       (g_OID_equal(ctx->mech_used, gss_mech_krb5_old) ?
 				ctx->seq->contents : NULL),
 			       md5cksum.contents, md5cksum.contents, 16))) {
-	    xfree(md5cksum.contents);
+	    krb5_free_checksum_contents(context, &md5cksum);
 	    if (toktype == KG_TOK_SEAL_MSG)
 		xfree(token.value);
 	    *minor_status = code;
@@ -284,7 +324,7 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
     case SGN_ALG_MD2_5:
 	if (!ctx->seed_init &&
 	    (code = kg_make_seed(context, ctx->subkey, ctx->seed))) {
-	    xfree(md5cksum.contents);
+	    krb5_free_checksum_contents(context, &md5cksum);
 	    if (sealalg != 0xffff)
 		xfree(plain);
 	    if (toktype == KG_TOK_SEAL_MSG)
@@ -296,7 +336,7 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 	if (! (data_ptr = (void *)
 	       xmalloc(sizeof(ctx->seed) + 8 +
 		       (ctx->big_endian ? token.length : plainlen)))) {
-	    xfree(md5cksum.contents);
+	    krb5_free_checksum_contents(context, &md5cksum);
 	    if (sealalg == 0)
 		xfree(plain);
 	    if (toktype == KG_TOK_SEAL_MSG)
@@ -315,9 +355,9 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 	plaind.length = 8 + sizeof(ctx->seed) +
 	    (ctx->big_endian ? token.length : plainlen);
 	plaind.data = data_ptr;
-	xfree(md5cksum.contents);
+	krb5_free_checksum_contents(context, &md5cksum);
 	code = krb5_c_make_checksum(context, md5cksum.checksum_type,
-				    ctx->seq, KG_USAGE_SIGN,
+				    ctx->seq, sign_usage,
 				    &plaind, &md5cksum);
 	xfree(data_ptr);
 
@@ -338,6 +378,7 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 	return(GSS_S_DEFECTIVE_TOKEN);
 
     case SGN_ALG_HMAC_SHA1_DES3_KD:
+    case SGN_ALG_HMAC_MD5:
 	/* compute the checksum of the message */
 
 	/* 8 = bytes of token body to be checksummed according to spec */
@@ -362,7 +403,7 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 	plaind.length = 8 + (ctx->big_endian ? token.length : plainlen);
 	plaind.data = data_ptr;
 	code = krb5_c_make_checksum(context, md5cksum.checksum_type,
-				    ctx->seq, KG_USAGE_SIGN,
+				    ctx->seq, sign_usage,
 				    &plaind, &md5cksum);
 	xfree(data_ptr);
 
@@ -373,11 +414,11 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 	    return(GSS_S_FAILURE);
 	}
 
-	code = memcmp(md5cksum.contents, ptr+14, md5cksum.length);
+	code = memcmp(md5cksum.contents, ptr+14, cksum_len);
 	break;
     }
 
-    xfree(md5cksum.contents);
+    krb5_free_checksum_contents(context, &md5cksum);
     if (sealalg != 0xffff)
 	xfree(plain);
 
@@ -414,14 +455,6 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
 
     /* do sequencing checks */
 
-    if ((code = kg_get_seq_num(context, ctx->seq, ptr+14, ptr+6, &direction,
-			       &seqnum))) {
-	if (toktype == KG_TOK_SEAL_MSG)
-	    xfree(token.value);
-	*minor_status = code;
-	return(GSS_S_BAD_SIG);
-    }
-
     if ((ctx->initiate && direction != 0xff) ||
 	(!ctx->initiate && direction != 0)) {
 	if (toktype == KG_TOK_SEAL_MSG)
@@ -442,9 +475,8 @@ kg_unseal_v1(context, minor_status, ctx, ptr, bodysize, message_buffer,
    conf_state is only valid if SEAL. */
 
 OM_uint32
-kg_unseal(context, minor_status, context_handle, input_token_buffer,
+kg_unseal(minor_status, context_handle, input_token_buffer,
 	  message_buffer, conf_state, qop_state, toktype)
-    krb5_context context;
     OM_uint32 *minor_status;
     gss_ctx_id_t context_handle;
     gss_buffer_t input_token_buffer;
@@ -455,8 +487,9 @@ kg_unseal(context, minor_status, context_handle, input_token_buffer,
 {
     krb5_gss_ctx_id_rec *ctx;
     unsigned char *ptr;
-    int bodysize;
+    unsigned int bodysize;
     int err;
+    int toktype2;
 
     /* validate the context handle */
     if (! kg_validate_ctx_id(context_handle)) {
@@ -477,14 +510,38 @@ kg_unseal(context, minor_status, context_handle, input_token_buffer,
 
     ptr = (unsigned char *) input_token_buffer->value;
 
-    if (!(err = g_verify_token_header((gss_OID) ctx->mech_used,
-				      &bodysize, &ptr, toktype,
-				      input_token_buffer->length))) {
-	return(kg_unseal_v1(context, minor_status, ctx, ptr, bodysize,
-			    message_buffer, conf_state, qop_state,
-			    toktype));
+    if (ctx->proto)
+	switch (toktype) {
+	case KG_TOK_SIGN_MSG:
+	    toktype2 = 0x0404;
+	    break;
+	case KG_TOK_SEAL_MSG:
+	    toktype2 = 0x0504;
+	    break;
+	case KG_TOK_DEL_CTX:
+	    toktype2 = 0x0405;
+	    break;
+	default:
+	    toktype2 = toktype;
+	    break;
+	}
+    else
+	toktype2 = toktype;
+    err = g_verify_token_header((gss_OID) ctx->mech_used,
+				&bodysize, &ptr, toktype2,
+				input_token_buffer->length,
+				!ctx->proto);
+    if (err) {
+	*minor_status = err;
+	return GSS_S_DEFECTIVE_TOKEN;
     }
 
-    *minor_status = err;
-    return(GSS_S_DEFECTIVE_TOKEN);
+    if (ctx->proto == 0)
+	return kg_unseal_v1(ctx->k5_context, minor_status, ctx, ptr, bodysize,
+			    message_buffer, conf_state, qop_state,
+			    toktype);
+    else
+	return gss_krb5int_unseal_token_v3(&ctx->k5_context, minor_status, ctx,
+					   ptr, bodysize, message_buffer,
+					   conf_state, qop_state, toktype);
 }
