@@ -72,10 +72,6 @@
 
 #include "gssapiP_krb5.h"
 
-/*
- * $Id: wrap_size_limit.c,v 1.7.6.2 2000/04/19 00:33:42 raeburn Exp $
- */
-
 /* V2 interface */
 OM_uint32
 krb5_gss_wrap_size_limit(minor_status, context_handle, conf_req_flag,
@@ -87,16 +83,11 @@ krb5_gss_wrap_size_limit(minor_status, context_handle, conf_req_flag,
     OM_uint32		req_output_size;
     OM_uint32		*max_input_size;
 {
-    krb5_context	context;
     krb5_gss_ctx_id_rec	*ctx;
-    krb5_error_code code;
     OM_uint32		data_size, conflen;
     OM_uint32		ohlen;
     int			overhead;
 
-    if (GSS_ERROR(kg_get_context(minor_status, &context)))
-       return(GSS_S_FAILURE);
-
     /* only default qop is allowed */
     if (qop_req != GSS_C_QOP_DEFAULT) {
 	*minor_status = (OM_uint32) G_UNKNOWN_QOP;
@@ -115,10 +106,44 @@ krb5_gss_wrap_size_limit(minor_status, context_handle, conf_req_flag,
 	return(GSS_S_NO_CONTEXT);
     }
 
+    if (ctx->proto == 1) {
+	/* No pseudo-ASN.1 wrapper overhead, so no sequence length and
+	   OID.  */
+	OM_uint32 sz = req_output_size;
+	/* Token header: 16 octets.  */
+	if (conf_req_flag) {
+	    while (sz > 0 && krb5_encrypt_size(sz, ctx->enc->enctype) + 16 > req_output_size)
+		sz--;
+	    /* Allow for encrypted copy of header.  */
+	    if (sz > 16)
+		sz -= 16;
+	    else
+		sz = 0;
+#ifdef CFX_EXERCISE
+	    /* Allow for EC padding.  In the MIT implementation, only
+	       added while testing.  */
+	    if (sz > 65535)
+		sz -= 65535;
+	    else
+		sz = 0;
+#endif
+	} else {
+	    /* Allow for token header and checksum.  */
+	    if (sz < 16 + ctx->cksum_size)
+		sz = 0;
+	    else
+		sz -= (16 + ctx->cksum_size);
+	}
+
+	*max_input_size = sz;
+	*minor_status = 0;
+	return GSS_S_COMPLETE;
+    }
+
     /* Calculate the token size and subtract that from the output size */
     overhead = 7 + ctx->mech_used->length;
     data_size = req_output_size;
-    conflen = kg_confounder_size(context, ctx->enc);
+    conflen = kg_confounder_size(ctx->k5_context, ctx->enc);
     data_size = (conflen + data_size + 8) & (~(OM_uint32)7);
     ohlen = g_token_size((gss_OID) ctx->mech_used,
 			 (unsigned int) (data_size + ctx->cksum_size + 14))
