@@ -10,9 +10,9 @@ static char *rcsid = "$Header$";
 
 #include	<sys/types.h>
 #include	<sys/time.h>
+#include	<errno.h>
 #include	<kadm5/admin.h>
 #include	"adb.h"
-#include	"k5-int.h"
 #include	<krb5/kdb.h>
 #include	<stdio.h>
 #include	<string.h>
