@@ -80,6 +80,9 @@ cleanup_key_data(context, count, data)
 {
     int i, j;
 
+    /* If data is NULL, count is always 0 */
+    if (data == NULL) return;
+
     for (i = 0; i < count; i++) {
 	for (j = 0; j < data[i].key_data_ver; j++) {
 	    if (data[i].key_data_length[j]) {
@@ -102,7 +105,6 @@ add_key_rnd(context, master_key, ks_tuple, ks_tuple_count, db_entry, kvno)
     krb5_principal	  krbtgt_princ;
     krb5_keyblock	  key;
     krb5_db_entry	  krbtgt_entry;
-    krb5_key_data	* krbtgt_kdata;
     krb5_boolean	  more;
     int			  max_kvno, one, i, j;
     krb5_error_code	  retval;
@@ -162,7 +164,7 @@ add_key_rnd(context, master_key, ks_tuple, ks_tuple_count, db_entry, kvno)
 	if (similar)
 	    continue;
 
-        if (retval = krb5_dbe_create_key_data(context, db_entry)) 
+        if ((retval = krb5_dbe_create_key_data(context, db_entry))) 
 	    goto add_key_rnd_err;
 
 	/* there used to be code here to extract the old key, and derive
@@ -276,8 +278,8 @@ krb5_dbe_ark(context, master_key, ks_tuple, ks_tuple_count, db_entry)
     /* increment the kvno */
     kvno++;
 
-    if (retval = add_key_rnd(context, master_key, ks_tuple, 
-			     ks_tuple_count, db_entry, kvno)) {
+    if ((retval = add_key_rnd(context, master_key, ks_tuple, 
+			     ks_tuple_count, db_entry, kvno))) {
 	cleanup_key_data(context, db_entry->n_key_data, db_entry->key_data);
 	db_entry->n_key_data = key_data_count;
 	db_entry->key_data = key_data;
@@ -285,7 +287,7 @@ krb5_dbe_ark(context, master_key, ks_tuple, ks_tuple_count, db_entry)
 	/* Copy keys with key_data_kvno == kvno - 1 ( = old kvno ) */
 	for (i = 0; i < key_data_count; i++) {
 	    if (key_data[i].key_data_kvno == (kvno - 1)) {
-		if (retval = krb5_dbe_create_key_data(context, db_entry)) {
+		if ((retval = krb5_dbe_create_key_data(context, db_entry))) {
 		    cleanup_key_data(context, db_entry->n_key_data,
 				     db_entry->key_data);
 		    break;
@@ -319,7 +321,6 @@ add_key_pwd(context, master_key, ks_tuple, ks_tuple_count, passwd,
     krb5_keysalt	  key_salt;
     krb5_keyblock	  key;
     krb5_data	  	  pwd;
-    krb5_boolean	  found;
     int			  i, j;
 
     retval = 0;
@@ -349,15 +350,15 @@ add_key_pwd(context, master_key, ks_tuple, ks_tuple_count, passwd,
 	if (j < i)
 	    continue;
 
-	if (retval = krb5_dbe_create_key_data(context, db_entry)) 
+	if ((retval = krb5_dbe_create_key_data(context, db_entry))) 
 	    return(retval);
 
 	/* Convert password string to key using appropriate salt */
 	switch (key_salt.type = ks_tuple[i].ks_salttype) {
     	case KRB5_KDB_SALTTYPE_ONLYREALM: {
             krb5_data * saltdata;
-            if (retval = krb5_copy_data(context, krb5_princ_realm(context,
-					db_entry->princ), &saltdata))
+            if ((retval = krb5_copy_data(context, krb5_princ_realm(context,
+					      db_entry->princ), &saltdata)))
 	 	return(retval);
 
 	    key_salt.data = *saltdata;
@@ -365,13 +366,13 @@ add_key_pwd(context, master_key, ks_tuple, ks_tuple_count, passwd,
 	}
 		break;
     	case KRB5_KDB_SALTTYPE_NOREALM:
-            if (retval=krb5_principal2salt_norealm(context, db_entry->princ,
-                                                         &key_salt.data)) 
+            if ((retval=krb5_principal2salt_norealm(context, db_entry->princ,
+						    &key_salt.data))) 
 		return(retval);
             break;
 	case KRB5_KDB_SALTTYPE_NORMAL:
-            if (retval = krb5_principal2salt(context, db_entry->princ,
-					         &key_salt.data)) 
+            if ((retval = krb5_principal2salt(context, db_entry->princ,
+					      &key_salt.data))) 
 		return(retval);
             break;
     	case KRB5_KDB_SALTTYPE_V4:
@@ -386,19 +387,20 @@ add_key_pwd(context, master_key, ks_tuple, ks_tuple_count, passwd,
 	 	return(retval);
 
 	    key_salt.data = *saltdata;
-	    key_salt.data.length = -1; /*length actually used below...*/
+	    key_salt.data.length = SALT_TYPE_AFS_LENGTH; /*length actually used below...*/
 	    krb5_xfree(saltdata);
 #else
 	    /* Why do we do this? Well, the afs_mit_string_to_key needs to
 	       use strlen, and the realm is not NULL terminated.... */
-	    int slen = (*krb5_princ_realm(context,db_entry->princ)).length;
+	    unsigned int slen = 
+		(*krb5_princ_realm(context,db_entry->princ)).length;
 	    if(!(key_salt.data.data = (char *) malloc(slen+1)))
 	        return ENOMEM;
 	    key_salt.data.data[slen] = 0;
 	    memcpy((char *)key_salt.data.data,
 		   (char *)(*krb5_princ_realm(context,db_entry->princ)).data,
 		   slen);
-	    key_salt.data.length = -1; /*length actually used below...*/
+	    key_salt.data.length = SALT_TYPE_AFS_LENGTH; /*length actually used below...*/
 #endif
 
 	}
@@ -410,6 +412,7 @@ add_key_pwd(context, master_key, ks_tuple, ks_tuple_count, passwd,
     	pwd.data = passwd;
     	pwd.length = strlen(passwd);
 
+	/* AFS string to key will happen here */
 	if ((retval = krb5_c_string_to_key(context, ks_tuple[i].ks_enctype,
 					   &pwd, &key_salt.data, &key))) {
 	     if (key_salt.data.data)
@@ -417,13 +420,13 @@ add_key_pwd(context, master_key, ks_tuple, ks_tuple_count, passwd,
 	     return(retval);
 	}
 
-	if (key_salt.data.length == -1)
+	if (key_salt.data.length == SALT_TYPE_AFS_LENGTH)
 	    key_salt.data.length = 
 	      krb5_princ_realm(context, db_entry->princ)->length;
 
-	if (retval = krb5_dbekd_encrypt_key_data(context, master_key, &key,
+	if ((retval = krb5_dbekd_encrypt_key_data(context, master_key, &key,
 		     (const krb5_keysalt *)&key_salt,
-		     kvno, &db_entry->key_data[db_entry->n_key_data-1])) {
+		     kvno, &db_entry->key_data[db_entry->n_key_data-1]))) {
 	    if (key_salt.data.data)
 		 free(key_salt.data.data);
 	    krb5_xfree(key.contents);
@@ -530,8 +533,8 @@ krb5_dbe_apw(context, master_key, ks_tuple, ks_tuple_count, passwd, db_entry)
     /* increment the kvno */
     new_kvno = old_kvno+1;
 
-    if (retval = add_key_pwd(context, master_key, ks_tuple, ks_tuple_count,
-			     passwd, db_entry, new_kvno)) {
+    if ((retval = add_key_pwd(context, master_key, ks_tuple, ks_tuple_count,
+			     passwd, db_entry, new_kvno))) {
 	cleanup_key_data(context, db_entry->n_key_data, db_entry->key_data);
 	db_entry->n_key_data = key_data_count;
 	db_entry->key_data = key_data;
@@ -539,7 +542,7 @@ krb5_dbe_apw(context, master_key, ks_tuple, ks_tuple_count, passwd, db_entry)
 	/* Copy keys with key_data_kvno == old_kvno */
 	for (i = 0; i < key_data_count; i++) {
 	    if (key_data[i].key_data_kvno == old_kvno) {
-		if (retval = krb5_dbe_create_key_data(context, db_entry)) {
+		if ((retval = krb5_dbe_create_key_data(context, db_entry))) {
 		    cleanup_key_data(context, db_entry->n_key_data,
 				     db_entry->key_data);
 		    break;
