@@ -16,7 +16,10 @@
  * this permission notice appear in supporting documentation, and that
  * the name of M.I.T. not be used in advertising or publicity pertaining
  * to distribution of the software without specific, written prior
- * permission.  M.I.T. makes no representations about the suitability of
+ * permission.  Furthermore if you modify this software you must label
+ * your software as modified software and not distribute it in such a
+ * fashion that it might be confused with the original M.I.T. software.
+ * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
  * 
@@ -51,11 +54,11 @@ extern char *progname;
 static char default_db_name[] = DEFAULT_KDB_FILE;
 
 static char *gen_dbsuffix 
-	PROTOTYPE((char *, char * ));
+	(char *, char * );
 static krb5_error_code krb5_dbm_db_start_update 
-	PROTOTYPE((krb5_context));
+	(krb5_context);
 static krb5_error_code krb5_dbm_db_end_update 
-	PROTOTYPE((krb5_context));
+	(krb5_context);
 
 /*
  * This module contains all of the code which directly interfaces to
@@ -330,10 +333,10 @@ krb5_dbm_db_close_database(context)
  * The should really reference the db_context
  */
 krb5_error_code
-krb5_dbm_db_set_mkey(context, db_context, eblock)
+krb5_dbm_db_set_mkey(context, db_context, key)
     krb5_context 	  context;
     krb5_db_context 	* db_context;
-    krb5_encrypt_block  * eblock;
+    krb5_keyblock  * key;
 {
     krb5_db_context *db_ctx;
 
@@ -341,15 +344,15 @@ krb5_dbm_db_set_mkey(context, db_context, eblock)
 	return(KRB5_KDB_DBNOTINITED);
 
     db_ctx = context->db_context;
-    db_ctx->db_master_key = eblock;
+    db_ctx->db_master_key = key;
     return 0;
 }
 
 krb5_error_code
-krb5_dbm_db_get_mkey(context, db_context, eblock)
+krb5_dbm_db_get_mkey(context, db_context, key)
     krb5_context 	  context;
     krb5_db_context 	* db_context;
-    krb5_encrypt_block  **eblock;
+    krb5_keyblock  **key;
 {
     krb5_db_context *db_ctx;
 
@@ -357,7 +360,7 @@ krb5_dbm_db_get_mkey(context, db_context, eblock)
 	return(KRB5_KDB_DBNOTINITED);
 
     db_ctx = context->db_context;
-    *eblock = db_ctx->db_master_key;
+    *key = db_ctx->db_master_key;
     return 0;
 
 }
@@ -606,7 +609,7 @@ krb5_dbm_db_create(context, db_name)
 /*
  * Destroy the database.  Zero's out all of the files, just to be sure.
  */
-krb5_error_code
+static krb5_error_code
 destroy_file_suffix(dbname, suffix)
 	char	*dbname;
 	char	*suffix;
@@ -930,7 +933,7 @@ errout:
 krb5_error_code
 krb5_dbm_db_get_principal(context, searchfor, entries, nentries, more)
     krb5_context context;
-krb5_principal searchfor;
+krb5_const_principal searchfor;
 krb5_db_entry *entries;		/* filled in */
 int *nentries;				/* how much room/how many found */
 krb5_boolean *more;			/* are there more? */
@@ -1060,7 +1063,7 @@ krb5_dbm_db_put_principal(context, entries, nentries)
 krb5_error_code
 krb5_dbm_db_delete_principal(context, searchfor, nentries)
     krb5_context 	  context;
-    krb5_principal 	  searchfor;
+    krb5_const_principal  searchfor;
     int 		* nentries;	/* how many found & deleted */
 {
     krb5_error_code 	  retval;
@@ -1129,7 +1132,7 @@ cleanup:
 krb5_error_code
 krb5_dbm_db_iterate (context, func, func_arg)
     krb5_context context;
-    krb5_error_code (*func) PROTOTYPE((krb5_pointer, krb5_db_entry *));
+    krb5_error_code (*func) (krb5_pointer, krb5_db_entry *);
     krb5_pointer func_arg;
 {
     datum key, contents;
