@@ -1,7 +1,7 @@
 /*
  * lib/krb5/ccache/ccbase.c
  *
- * Copyright 1990 by the Massachusetts Institute of Technology.
+ * Copyright 1990,2004 by the Massachusetts Institute of Technology.
  * All Rights Reserved.
  *
  * Export of this software from the United States of America may
@@ -28,14 +28,15 @@
  */
 
 #include "k5-int.h"
+#include "k5-thread.h"
 
 #include "fcc.h"
+#include "cc-int.h"
 
-struct krb5_cc_typelist
- {
-  krb5_cc_ops *ops;
-  struct krb5_cc_typelist *next;
- };
+struct krb5_cc_typelist {
+    const krb5_cc_ops *ops;
+    struct krb5_cc_typelist *next;
+};
 extern const krb5_cc_ops krb5_mcc_ops;
 
 #ifdef _WIN32
@@ -50,6 +51,37 @@ static struct krb5_cc_typelist cc_fcc_entry = { &krb5_cc_file_ops,
 						&cc_mcc_entry };
 
 static struct krb5_cc_typelist *cc_typehead = &cc_fcc_entry;
+static k5_mutex_t cc_typelist_lock = K5_MUTEX_PARTIAL_INITIALIZER;
+
+int
+krb5int_cc_initialize(void)
+{
+    int err;
+
+    err = k5_mutex_finish_init(&krb5int_mcc_mutex);
+    if (err)
+	return err;
+    err = k5_mutex_finish_init(&cc_typelist_lock);
+    if (err)
+	return err;
+    err = k5_mutex_finish_init(&krb5int_cc_file_mutex);
+    if (err)
+	return err;
+    return 0;
+}
+
+void
+krb5int_cc_finalize(void)
+{
+    struct krb5_cc_typelist *t, *t_next;
+    k5_mutex_destroy(&cc_typelist_lock);
+    k5_mutex_destroy(&krb5int_cc_file_mutex);
+    k5_mutex_destroy(&krb5int_mcc_mutex);
+    for (t = cc_typehead; t != &cc_fcc_entry; t = t_next) {
+	t_next = t->next;
+	free(t);
+    }
+}
 
 
 /*
@@ -61,20 +93,31 @@ krb5_error_code KRB5_CALLCONV
 krb5_cc_register(krb5_context context, krb5_cc_ops *ops, krb5_boolean override)
 {
     struct krb5_cc_typelist *t;
+    krb5_error_code err;
+
+    err = k5_mutex_lock(&cc_typelist_lock);
+    if (err)
+	return err;
     for (t = cc_typehead;t && strcmp(t->ops->prefix,ops->prefix);t = t->next)
 	;
     if (t) {
 	if (override) {
 	    t->ops = ops;
+	    k5_mutex_unlock(&cc_typelist_lock);
 	    return 0;
-	} else
+	} else {
+	    k5_mutex_unlock(&cc_typelist_lock);
 	    return KRB5_CC_TYPE_EXISTS;
+	}
     }
-    if (!(t = (struct krb5_cc_typelist *) malloc(sizeof(*t))))
+    if (!(t = (struct krb5_cc_typelist *) malloc(sizeof(*t)))) {
+	k5_mutex_unlock(&cc_typelist_lock);
 	return ENOMEM;
+    }
     t->next = cc_typehead;
     t->ops = ops;
     cc_typehead = t;
+    k5_mutex_unlock(&cc_typelist_lock);
     return 0;
 }
 
@@ -88,6 +131,7 @@ krb5_cc_register(krb5_context context, krb5_cc_ops *ops, krb5_boolean override)
  * particular cache type.
  */
 
+#include <ctype.h>
 krb5_error_code KRB5_CALLCONV
 krb5_cc_resolve (krb5_context context, const char *name, krb5_ccache *cache)
 {
@@ -95,6 +139,7 @@ krb5_cc_resolve (krb5_context context, const char *name, krb5_ccache *cache)
     char *pfx, *cp;
     const char *resid;
     unsigned int pfxlen;
+    krb5_error_code err;
     
     cp = strchr (name, ':');
     if (!cp) {
@@ -105,11 +150,6 @@ krb5_cc_resolve (krb5_context context, const char *name, krb5_ccache *cache)
     }
 
     pfxlen = cp - name;
-    resid = name + pfxlen + 1;
-	
-    pfx = malloc (pfxlen+1);
-    if (!pfx)
-	return ENOMEM;
 
     if ( pfxlen == 1 && isalpha(name[0]) ) {
         /* We found a drive letter not a prefix - use FILE: */
@@ -124,18 +164,27 @@ krb5_cc_resolve (krb5_context context, const char *name, krb5_ccache *cache)
         pfx = malloc (pfxlen+1);
         if (!pfx)
             return ENOMEM;
+
         memcpy (pfx, name, pfxlen);
         pfx[pfxlen] = '\0';
     }
 
     *cache = (krb5_ccache) 0;
 
+    err = k5_mutex_lock(&cc_typelist_lock);
+    if (err) {
+	free(pfx);
+	return err;
+    }
     for (tlist = cc_typehead; tlist; tlist = tlist->next) {
 	if (strcmp (tlist->ops->prefix, pfx) == 0) {
+	    krb5_error_code (KRB5_CALLCONV *ccresolver)() = tlist->ops->resolve;
+	    k5_mutex_unlock(&cc_typelist_lock);
 	    free(pfx);
-	    return (*tlist->ops->resolve)(context, cache, resid);
+	    return (*ccresolver)(context, cache, resid);
 	}
     }
+    k5_mutex_unlock(&cc_typelist_lock);
     if (krb5_cc_dfl_ops && !strcmp (pfx, krb5_cc_dfl_ops->prefix)) {
 	free (pfx);
 	return (*krb5_cc_dfl_ops->resolve)(context, cache, resid);
