@@ -38,6 +38,7 @@
 #if	HAVE_REGEX_H
 #include <regex.h>
 #endif	/* HAVE_REGEX_H */
+#include <string.h>
 /*
  * Use compile(3) if no regcomp present.
  */
@@ -100,7 +101,7 @@ db_an_to_ln(context, dbname, aname, lnsize, lname)
     const unsigned int lnsize;
     char *lname;
 {
-#if	(!defined(_WIN32) && !defined(macintosh))
+#if !defined(_WIN32)
     DBM *db;
     krb5_error_code retval;
     datum key, contents;
