@@ -36,6 +36,7 @@
 #endif
 
 #if defined(_WIN32)
+#include <winsock.h>
 
 static krb5_error_code
 get_from_windows_dir(
@@ -342,33 +343,35 @@ krb5_os_init_context(krb5_context ctx)
 {
 	krb5_os_context os_ctx;
 	krb5_error_code	retval = 0;
+#ifdef _WIN32
+    WORD wVersionRequested;
+    WSADATA wsaData;
+#endif /* _WIN32 */
 
-	if (ctx->os_context)
-		return 0;
-
-	os_ctx = malloc(sizeof(struct _krb5_os_context));
-	if (!os_ctx)
-		return ENOMEM;
-	memset(os_ctx, 0, sizeof(struct _krb5_os_context));
+	os_ctx = ctx->os_context;
 	os_ctx->magic = KV5M_OS_CONTEXT;
-
-	ctx->os_context = (void *) os_ctx;
-
 	os_ctx->time_offset = 0;
 	os_ctx->usec_offset = 0;
 	os_ctx->os_flags = 0;
 	os_ctx->default_ccname = 0;
-	os_ctx->default_ccprincipal = 0;
-
-	krb5_cc_set_default_name(ctx, NULL);
 
 	retval = os_init_paths(ctx);
-
 	/*
 	 * If there's an error in the profile, return an error.  Just
 	 * ignoring the error is a Bad Thing (tm).
 	 */
+     
+        if (!retval) {
+                krb5_cc_set_default_name(ctx, NULL);
 
+#ifdef _WIN32
+                /* We initialize winsock to version 1.1 but 
+                 * we do not care if we succeed or fail.
+                 */
+                wVersionRequested = 0x0101;
+                WSAStartup (wVersionRequested, &wsaData);
+#endif /* _WIN32 */
+        }
 	return retval;
 }
 
@@ -463,25 +466,19 @@ krb5_os_free_context(krb5_context ctx)
 
 	os_ctx = ctx->os_context;
 	
-	if (!os_ctx)
-		return;
-
 	if (os_ctx->default_ccname) {
 		free(os_ctx->default_ccname);
                 os_ctx->default_ccname = 0;
         }
 
-	if (os_ctx->default_ccprincipal) {
-		krb5_free_principal (ctx, os_ctx->default_ccprincipal);
-		os_ctx->default_ccprincipal = 0;
-	}
-
 	os_ctx->magic = 0;
-	free(os_ctx);
-	ctx->os_context = 0;
 
 	if (ctx->profile) {
 		profile_release(ctx->profile);
 	    ctx->profile = 0;
 	}
+
+#ifdef _WIN32
+        WSACleanup();
+#endif /* _WIN32 */
 }
