@@ -184,7 +184,7 @@
  * non-root.
  */
 
-#include <com_err.h>
+#include "com_err.h"
 #include "libpty.h"
 #include "pty-int.h"
 #include <sys/wait.h>
