@@ -4,13 +4,12 @@
  * For copyright information, see copyright.h.
  */
 
-#include <stdlib.h>
-
 #include "copyright.h"
 #include "ss_internal.h"
 
 #define ssrt ss_request_table	/* for some readable code... */
 
+void
 ss_add_request_table(sci_idx, rqtbl_ptr, position, code_ptr)
 	int sci_idx;
 	ssrt *rqtbl_ptr;
@@ -25,8 +24,8 @@ ss_add_request_table(sci_idx, rqtbl_ptr, position, code_ptr)
 		;
 	/* size == C subscript of NULL == #elements */
 	size += 2;		/* new element, and NULL */
-	info->rqt_tables = (ssrt **)realloc((char *)info->rqt_tables,
-					    (unsigned)size*sizeof(ssrt));
+	info->rqt_tables = (ssrt **)realloc(info->rqt_tables,
+					    size*sizeof(ssrt));
 	if (info->rqt_tables == (ssrt **)NULL) {
 		*code_ptr = errno;
 		return;
@@ -43,6 +42,7 @@ ss_add_request_table(sci_idx, rqtbl_ptr, position, code_ptr)
 	*code_ptr = 0;
 }
 
+void
 ss_delete_request_table(sci_idx, rqtbl_ptr, code_ptr)
      int sci_idx;
      ssrt *rqtbl_ptr;
