@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Massachusetts Institute of Technology
+ * Copyright (c) 2005 Massachusetts Institute of Technology
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -42,7 +42,7 @@
 
     \see kconf_load_schema()
 */
-typedef struct kconf_schema_t {
+typedef struct tag_kconf_schema {
     wchar_t *   name;       /*!< name of the object being described.
                                 Optional for KC_ENDSPACE type object,
                                 but required for everything else.
@@ -63,7 +63,7 @@ typedef struct kconf_schema_t {
                                 not support defining a default value
                                 here. */
     wchar_t *   description;/*!< a friendly description of the value
-                                or configuration space */
+                                or configuration space. */
 } kconf_schema;
 
 /*! \name Configuration data types
@@ -143,6 +143,31 @@ typedef struct kconf_schema_t {
 /*! \brief Indicates that the last component of the given configuration path is to be considered to be a configuration value */
 #define KCONF_FLAG_TRAILINGVALUE 0x00000020
 
+/*! \brief Only write values back there is a change
+
+    Any write operations using the handle with check if the value
+    being written is different from the value being read from the
+    handle.  It will only be written if the value is different.
+
+    \note Note that the value being read from a handle takes schema and
+    shadowed configuration handles into consideration while the value
+    being written is only written to the topmost layer of
+    configuration that can be written to.
+
+    \note Note also that this flag does not affect binary values.
+ */
+#define KCONF_FLAG_WRITEIFMOD    0x00000040
+
+/*! \brief Use case-insensitive comparison for KCONF_FLAG_WRITEIFMOD
+
+    When used in combination with \a KCONF_FLAG_WRITEIFMOD , the
+    string comparison used when determining whether the string read
+    from the configuration handle is the same as the string being
+    written will be case insensitive.  If this flag is not set, the
+    comparison will be case sensitive.
+ */
+#define KCONF_FLAG_IFMODCI     0x00000080
+
 /*! \brief Do not parse the configuration space name
 
     If set, disables the parsing of the configuration space for
@@ -153,12 +178,12 @@ typedef struct kconf_schema_t {
     By default, the configuration space name,
 
     \code
-    L"foo/bar"
+    L"foo\\bar"
     \endcode
 
     is taken to mean the configuration space \a bar which is a
     subspace of \a foo.  If ::KCONF_FLAG_NOPARSENAME is set, then this
-    is taken to mean configuration space \a foo/bar.
+    is taken to mean configuration space \a foo\\bar.
  */
 #define KCONF_FLAG_NOPARSENAME   0x00000040
 
@@ -228,7 +253,9 @@ typedef struct kconf_schema_t {
         and settings \a flags to the required flags.
 
 */
-KHMEXP khm_int32 KHMAPI khc_open_space(khm_handle parent, wchar_t * cspace, khm_int32 flags, khm_handle * result);
+KHMEXP khm_int32 KHMAPI 
+khc_open_space(khm_handle parent, const wchar_t * cspace, khm_int32 flags, 
+               khm_handle * result);
 
 /*! \brief Set the shadow space for a configuration handle
 
@@ -250,15 +277,17 @@ KHMEXP khm_int32 KHMAPI khc_open_space(khm_handle parent, wchar_t * cspace, khm_
 
     Specify NULL for \a lower to remove any prior shadow.
  */
-KHMEXP khm_int32 KHMAPI khc_shadow_space(khm_handle upper, khm_handle lower);
+KHMEXP khm_int32 KHMAPI 
+khc_shadow_space(khm_handle upper, khm_handle lower);
 
 /*! \brief Close a handle opened with khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_close_space(khm_handle conf);
+KHMEXP khm_int32 KHMAPI 
+khc_close_space(khm_handle conf);
 
 /*! \brief Read a string value from a configuration space
 
-    The \a value parameter specifies the value to read from the
+    The \a value_name parameter specifies the value to read from the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -286,6 +315,10 @@ KHMEXP khm_int32 KHMAPI khc_close_space(khm_handle conf);
     in the call to khc_open_space() is equivalent to specifying all
     three.
 
+    If the value is not found in the configuration space and any
+    shadowed configuration spaces, the function returns \a
+    KHM_ERROR_NOT_FOUND.  In this case, the buffer is left unmodified.
+
     \param[in] buf Buffer to copy the string to.  Specify NULL to just
         retrieve the number of required bytes.
     
@@ -295,22 +328,23 @@ KHMEXP khm_int32 KHMAPI khc_close_space(khm_handle conf);
         the required buffer if \a buf is NULL or insufficient.
 
     \retval KHM_ERROR_NOT_READY The configuration provider has not started
-    \retval KHM_ERROR_INVALID_PARM One or more of the supplied parameters are not valid
+    \retval KHM_ERROR_INVALID_PARAM One or more of the supplied parameters are not valid
     \retval KHM_ERROR_TYPE_MISMATCH The specified value is not a string
     \retval KHM_ERROR_TOO_LONG \a buf was NULL or the size of the buffer was insufficient.  The required size is in bufsize.
     \retval KHM_ERROR_SUCCESS Success.  The number of bytes copied is in bufsize.
+    \retval KHM_ERROR_NOT_FOUND The value was not found.
 
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_read_string(
-    khm_handle conf, 
-    wchar_t * value, 
-    wchar_t * buf, 
-    khm_size * bufsize);
+KHMEXP khm_int32 KHMAPI 
+khc_read_string(khm_handle conf, 
+                const wchar_t * value_name, 
+                wchar_t * buf, 
+                khm_size * bufsize);
 
 /*! \brief Read a multi-string value from a configuration space
 
-    The \a value parameter specifies the value to read from the
+    The \a value_name parameter specifies the value to read from the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -346,6 +380,10 @@ KHMEXP khm_int32 KHMAPI khc_read_string(
     in the call to khc_open_space() is equivalent to specifying all
     three.
 
+    If the value is not found in the configuration space and any
+    shadowed configuration spaces, the function returns \a
+    KHM_ERROR_NOT_FOUND.  In this case, the buffer is left unmodified.
+
     \param[in] buf Buffer to copy the multi-string to.  Specify NULL
         to just retrieve the number of required bytes.
     
@@ -355,22 +393,23 @@ KHMEXP khm_int32 KHMAPI khc_read_string(
         the required buffer if \a buf is NULL or insufficient.
 
     \retval KHM_ERROR_NOT_READY The configuration provider has not started
-    \retval KHM_ERROR_INVALID_PARM One or more of the supplied parameters are not valid
+    \retval KHM_ERROR_INVALID_PARAM One or more of the supplied parameters are not valid
     \retval KHM_ERROR_TYPE_MISMATCH The specified value is not a string
     \retval KHM_ERROR_TOO_LONG \a buf was NULL or the size of the buffer was insufficient.  The required size is in bufsize.
     \retval KHM_ERROR_SUCCESS Success.  The number of bytes copied is in bufsize.
+    \retval KHM_ERROR_NOT_FOUND The value was not found.
 
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_read_multi_string(
-    khm_handle conf, 
-    wchar_t * value, 
-    wchar_t * buf, 
-    khm_size * bufsize);
+KHMEXP khm_int32 KHMAPI 
+khc_read_multi_string(khm_handle conf, 
+                      const wchar_t * value_name, 
+                      wchar_t * buf, 
+                      khm_size * bufsize);
 
 /*! \brief Read a 32 bit integer value from a configuration space
 
-    The \a value parameter specifies the value to read from the
+    The \a value_name parameter specifies the value to read from the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -398,6 +437,10 @@ KHMEXP khm_int32 KHMAPI khc_read_multi_string(
     in the call to khc_open_space() is equivalent to specifying all
     three.
 
+    If the value is not found in the configuration space and any
+    shadowed configuration spaces, the function returns \a
+    KHM_ERROR_NOT_FOUND.  In this case, the buffer is left unmodified.
+
     \param[in] conf Handle to a configuration space
     \param[in] value The value to query
     \param[out] buf The buffer to receive the value
@@ -405,18 +448,18 @@ KHMEXP khm_int32 KHMAPI khc_read_multi_string(
     \retval KHM_ERROR_NOT_READY The configuration provider has not started.
     \retval KHM_ERROR_SUCCESS Success.  The value that was read was placed in \a buf
     \retval KHM_ERROR_NOT_FOUND The specified value was not found
-    \retval KHM_ERROR_INVALID_PARM One or more parameters were invalid
+    \retval KHM_ERROR_INVALID_PARAM One or more parameters were invalid
     \retval KHM_ERROR_TYPE_MISMATCH The specified value was found but was not of the correct type.
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_read_int32(
-    khm_handle conf, 
-    wchar_t * value, 
-    khm_int32 * buf);
+KHMEXP khm_int32 KHMAPI 
+khc_read_int32(khm_handle conf, 
+               const wchar_t * value_name, 
+               khm_int32 * buf);
 
 /*! \brief Read a 64 bit integer value from a configuration space
 
-    The \a value parameter specifies the value to read from the
+    The \a value_name parameter specifies the value to read from the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -444,26 +487,30 @@ KHMEXP khm_int32 KHMAPI khc_read_int32(
     in the call to khc_open_space() is equivalent to specifying all
     three.
 
+    If the value is not found in the configuration space and any
+    shadowed configuration spaces, the function returns \a
+    KHM_ERROR_NOT_FOUND.  In this case, the buffer is left unmodified.
+
     \param[in] conf Handle to a configuration space
-    \param[in] value The value to query
+    \param[in] value_name The value to query
     \param[out] buf The buffer to receive the value
 
     \retval KHM_ERROR_NOT_READY The configuration provider has not started
     \retval KHM_ERROR_SUCCESS Success.  The value that was read was placed in \a buf
     \retval KHM_ERROR_NOT_FOUND The specified value was not found
-    \retval KHM_ERROR_INVALID_PARM One or more parameters were invalid
+    \retval KHM_ERROR_INVALID_PARAM One or more parameters were invalid
     \retval KHM_ERROR_TYPE_MISMATCH The specified value was found but was not the correct data type.
 
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_read_int64(
-    khm_handle conf, 
-    wchar_t * value, 
-    khm_int64 * buf);
+KHMEXP khm_int32 KHMAPI 
+khc_read_int64(khm_handle conf, 
+               const wchar_t * value_name, 
+               khm_int64 * buf);
 
 /*! \brief Read a binary value from a configuration space
 
-    The \a value parameter specifies the value to read from the
+    The \a value_name parameter specifies the value to read from the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -489,6 +536,10 @@ KHMEXP khm_int32 KHMAPI khc_read_int64(
     three. Also note that the schema store (KCONF_FLAG_SCHEMA) does
     not support binary values.
 
+    If the value is not found in the configuration space and any
+    shadowed configuration spaces, the function returns \a
+    KHM_ERROR_NOT_FOUND.  In this case, the buffer is left unmodified.
+
     \param[in] buf Buffer to copy the string to.  Specify NULL to just
         retrieve the number of required bytes.
     
@@ -499,19 +550,19 @@ KHMEXP khm_int32 KHMAPI khc_read_int64(
 
     \retval KHM_ERROR_SUCCESS Success. The data was copied to \a buf.  The number of bytes copied is stored in \a bufsize
     \retval KHM_ERROR_NOT_FOUND The specified value was not found
-    \retval KHM_ERROR_INVALID_PARM One or more parameters were invalid.
+    \retval KHM_ERROR_INVALID_PARAM One or more parameters were invalid.
 
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_read_binary(
-    khm_handle conf, 
-    wchar_t * value, 
-    void * buf, 
-    khm_size * bufsize);
+KHMEXP khm_int32 KHMAPI 
+khc_read_binary(khm_handle conf, 
+                const wchar_t * value_name, 
+                void * buf, 
+                khm_size * bufsize);
 
 /*! \brief Write a string value to a configuration space
 
-    The \a value parameter specifies the value to write to the
+    The \a value_name parameter specifies the value to write to the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -526,31 +577,40 @@ KHMEXP khm_int32 KHMAPI khc_read_binary(
     khc_open_space().  The precedence of configuration stores are as
     follows:
 
-    - If KCONF_FLAG_USER was specified, then the user configuration
+    - If \a KCONF_FLAG_USER was specified, then the user configuration
       space.
 
-    - Otherwise, if KCONF_FLAG_MACHINE was specified, then the machine
-      configuration space.
+    - Otherwise, if \a KCONF_FLAG_MACHINE was specified, then the
+      machine configuration space.
 
     Note that not specifying any of the configuration store specifiers
     in the call to khc_open_space() is equivalent to specifying all
     three.  Also note that the schema store (KCONF_FLAG_SCHEMA) is
     readonly.
 
+    If the \a KCONF_FLAG_WRITEIFMOD flag is specified in the call to
+    khc_open_space() for obtaining the configuration handle, the
+    specified string will only be written if it is different from the
+    value being read from the handle.
+
+    If the \a KCONF_FLAG_IFMODCI flag is specified along with the \a
+    KCONF_FLAG_WRITEIFMOD flag, then the string comparison used will
+    be case insensitive.
+
     \param[in] conf Handle to a configuration space
-    \param[in] value Name of value to write
+    \param[in] value_name Name of value to write
     \param[in] buf A NULL terminated unicode string not exceeding KCONF_MAXCCH_STRING in characters including terminating NULL
 
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_write_string(
-    khm_handle conf, 
-    wchar_t * value, 
-    wchar_t * buf);
+KHMEXP khm_int32 KHMAPI 
+khc_write_string(khm_handle conf, 
+                 const wchar_t * value_name, 
+                 wchar_t * buf);
 
 /*! \brief Write a multi-string value to a configuration space
 
-    The \a value parameter specifies the value to write to the
+    The \a value_name parameter specifies the value to write to the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -582,16 +642,25 @@ KHMEXP khm_int32 KHMAPI khc_write_string(
     three.  Also note that the schema store (KCONF_FLAG_SCHEMA) is
     readonly.
 
+    If the \a KCONF_FLAG_WRITEIFMOD flag is specified in the call to
+    khc_open_space() for obtaining the configuration handle, the
+    specified string will only be written if it is different from the
+    value being read from the handle.
+
+    If the \a KCONF_FLAG_IFMODCI flag is specified along with the \a
+    KCONF_FLAG_WRITEIFMOD flag, then the string comparison used will
+    be case insensitive.
+
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_write_multi_string(
-    khm_handle conf, 
-    wchar_t * value, 
-    wchar_t * buf);
+KHMEXP khm_int32 KHMAPI 
+khc_write_multi_string(khm_handle conf, 
+                       const wchar_t * value_name, 
+                       wchar_t * buf);
 
 /*! \brief Write a 32 bit integer value to a configuration space
 
-    The \a value parameter specifies the value to write to the
+    The \a value_name parameter specifies the value to write to the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -617,16 +686,21 @@ KHMEXP khm_int32 KHMAPI khc_write_multi_string(
     three.  Also note that the schema store (KCONF_FLAG_SCHEMA) is
     readonly.
 
+    If the \a KCONF_FLAG_WRITEIFMOD flag is specified in the call to
+    khc_open_space() for obtaining the configuration handle, the
+    specified string will only be written if it is different from the
+    value being read from the handle.
+
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_write_int32(
-    khm_handle conf, 
-    wchar_t * value, 
-    khm_int32 buf);
+KHMEXP khm_int32 KHMAPI 
+khc_write_int32(khm_handle conf, 
+                const wchar_t * value_name, 
+                khm_int32 buf);
 
 /*! \brief Write a 64 bit integer value to a configuration space
 
-    The \a value parameter specifies the value to write to the
+    The \a value_name parameter specifies the value to write to the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -652,16 +726,21 @@ KHMEXP khm_int32 KHMAPI khc_write_int32(
     three.  Also note that the schema store (KCONF_FLAG_SCHEMA) is
     readonly.
 
+    If the \a KCONF_FLAG_WRITEIFMOD flag is specified in the call to
+    khc_open_space() for obtaining the configuration handle, the
+    specified string will only be written if it is different from the
+    value being read from the handle.
+
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_write_int64(
-    khm_handle conf, 
-    wchar_t * value, 
-    khm_int64 buf);
+KHMEXP khm_int32 KHMAPI 
+khc_write_int64(khm_handle conf, 
+                const wchar_t * value_name, 
+                khm_int64 buf);
 
 /*! \brief Write a binary value to a configuration space
 
-    The \a value parameter specifies the value to write to the
+    The \a value_name parameter specifies the value to write to the
     configuration space.  This can be either a value name or a value
     path consisting of a series nested configuration space names
     followed by the value name all separated by backslashes or forward
@@ -689,18 +768,19 @@ KHMEXP khm_int32 KHMAPI khc_write_int64(
 
     \see khc_open_space()
 */
-KHMEXP khm_int32 KHMAPI khc_write_binary(
-    khm_handle conf, 
-    wchar_t * value, 
-    void * buf, 
-    khm_size bufsize);
+KHMEXP khm_int32 KHMAPI 
+khc_write_binary(khm_handle conf, 
+                 const wchar_t * value_name, 
+                 void * buf, 
+                 khm_size bufsize);
 
 /*! \brief Get the type of a value in a configuration space
 
     \return The return value is the type of the specified value, or
         KC_NONE if the value does not exist.
  */
-KHMEXP khm_int32 KHMAPI khc_get_type(khm_handle conf, wchar_t * value);
+KHMEXP khm_int32 KHMAPI 
+khc_get_type(khm_handle conf, const wchar_t * value_name);
 
 /*! \brief Check which configuration stores contain a specific value.
 
@@ -716,7 +796,49 @@ KHMEXP khm_int32 KHMAPI khc_get_type(khm_handle conf, wchar_t * value);
         and ::KCONF_FLAG_SCHEMA indicating which stores contain the
         value.
  */
-KHMEXP khm_int32 KHMAPI khc_value_exists(khm_handle conf, wchar_t * value);
+KHMEXP khm_int32 KHMAPI 
+khc_value_exists(khm_handle conf, const wchar_t * value);
+
+/*! \brief Remove a value from a configuration space
+
+    Removes a value from one or more configuration stores.
+
+    A value can exist in multiple configuration stores.  Only the
+    values that are stored in writable stores can be removed.  When
+    the function searches for values to remove, it will only look in
+    configuration stores that are specified in the handle.  In
+    addition, the configuration stores affected can be further
+    narrowed by specifying them in the \a flags parameter.  If \a
+    flags is zero, then all the stores visible to the handle are
+    searched.  If \a flags specifies ::KCONF_FLAG_USER or
+    ::KCONF_FLAG_MACHINE or both, then only the specified stores are
+    searched, provided that the stores are visible to the handle.
+
+    This function only operates on the topmost configuration space
+    visible to the handle.  If the configuration handle is shadowed,
+    the shadowed configuration spaces are unaffected by the removal.
+
+    \param[in] conf Handle to configuration space to remove value from
+
+    \param[in] value_name Value to remove
+
+    \param[in] flags Specifies which configuration stores will be
+        affected by the removal.  See above.
+
+    \retval KHM_ERROR_SUCCESS The value was removed from all the
+        specified configuration stores.
+
+    \retval KHM_ERROR_NOT_FOUND The value was not found.
+
+    \retval KHM_ERROR_UNKNOWN An unknown error occurred while trying
+        to remove the value.
+
+    \retval KHM_ERROR_PARTIAL The value was successfully removed from
+        one or more stores, but the operation failed on one or more
+        other stores.
+ */
+KHMEXP khm_int32 KHMAPI
+khc_remove_value(khm_handle conf, const wchar_t * value_name, khm_int32 flags);
 
 /*! \brief Get the name of a configuration space
 
@@ -729,10 +851,10 @@ KHMEXP khm_int32 KHMAPI khc_value_exists(khm_handle conf, wchar_t * value);
         pointed to by \a buf.  On exit, holds the number of bytes
         copied into the buffer including the NULL terminator.
  */
-KHMEXP khm_int32 KHMAPI khc_get_config_space_name(
-    khm_handle conf, 
-    wchar_t * buf, 
-    khm_size * bufsize);
+KHMEXP khm_int32 KHMAPI 
+khc_get_config_space_name(khm_handle conf, 
+                          wchar_t * buf, 
+                          khm_size * bufsize);
 
 /*! \brief Get a handle to the parent space
 
@@ -742,9 +864,9 @@ KHMEXP khm_int32 KHMAPI khc_get_config_space_name(
         call succeeds.  Receives NULL otherwise.  The returned handle
         must be closed using khc_close_space()
  */
-KHMEXP khm_int32 KHMAPI khc_get_config_space_parent(
-    khm_handle conf, 
-    khm_handle * parent);
+KHMEXP khm_int32 KHMAPI 
+khc_get_config_space_parent(khm_handle conf, 
+                            khm_handle * parent);
 
 /*! \brief Load a configuration schema into the specified configuration space
 
@@ -756,15 +878,15 @@ KHMEXP khm_int32 KHMAPI khc_get_config_space_parent(
 
     \see khc_unload_schema()
  */
-KHMEXP khm_int32 KHMAPI khc_load_schema(
-    khm_handle conf, 
-    kconf_schema * schema);
+KHMEXP khm_int32 KHMAPI 
+khc_load_schema(khm_handle conf, 
+                const kconf_schema * schema);
 
 /*! \brief Unload a schema from a configuration space
  */
-KHMEXP khm_int32 KHMAPI khc_unload_schema(
-    khm_handle conf, 
-    kconf_schema * schema);
+KHMEXP khm_int32 KHMAPI 
+khc_unload_schema(khm_handle conf, 
+                  const kconf_schema * schema);
 
 /*! \brief Enumerate the subspaces of a configuration space
 
@@ -788,7 +910,7 @@ KHMEXP khm_int32 KHMAPI khc_unload_schema(
     \retval KHM_ERROR_SUCCESS The call succeeded.  There is a valid
         handle to a configuration space in \a first_subspace.
 
-    \retval KHM_ERROR_INVALID_PARM Either \a conf or \a prev was not a
+    \retval KHM_ERROR_INVALID_PARAM Either \a conf or \a prev was not a
         valid configuration space handle or \a first_subspace is NULL.
         Note that \a prev can be NULL.
 
@@ -813,11 +935,20 @@ KHMEXP khm_int32 KHMAPI khc_unload_schema(
 	However, the returned handle has the same domain restrictions
 	as \a conf.
  */
-KHMEXP khm_int32 KHMAPI khc_enum_subspaces(
-    khm_handle conf,
-    khm_handle prev,
-    khm_handle * next);
+KHMEXP khm_int32 KHMAPI 
+khc_enum_subspaces(khm_handle conf,
+                   khm_handle prev,
+                   khm_handle * next);
 
+/*! \brief Remove a configuration space
+
+    The configuration space will be marked for removal.  Once all the
+    handles for the space have been released, it will be deleted.  The
+    configuration stores that will be affected are the write enabled
+    configuration stores for the handle.
+ */
+KHMEXP khm_int32 KHMAPI
+khc_remove_space(khm_handle conf);
 /*@}*/
 
 #endif
