@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Massachusetts Institute of Technology
+ * Copyright (c) 2005 Massachusetts Institute of Technology
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -59,11 +59,11 @@ void kcdb_cred_exit(void)
     can be called by kcdb_cred_dup with a write lock on l_creds and in other
     places with a read lock on l_creds.  New credentials must be creatable while
     holding either lock. */
-KHMEXP khm_int32 KHMAPI kcdb_cred_create(
-    wchar_t *   name, 
-    khm_handle  identity,
-    khm_int32   cred_type,
-    khm_handle * result) 
+KHMEXP khm_int32 KHMAPI 
+kcdb_cred_create(wchar_t *   name, 
+                 khm_handle  identity,
+                 khm_int32   cred_type,
+                 khm_handle * result) 
 {
     kcdb_cred * cred;
     size_t cb_name;
@@ -71,20 +71,18 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_create(
     if(!name || !result ||
         FAILED(StringCbLength(name, KCDB_CRED_MAXCB_NAME, &cb_name)) ||
         KHM_FAILED(kcdb_credtype_get_info(cred_type, NULL)) ||
-        KHM_FAILED(kcdb_identity_hold(identity))
-        )
-    {
-        return KHM_ERROR_INVALID_PARM;
+        KHM_FAILED(kcdb_identity_hold(identity))) {
+        return KHM_ERROR_INVALID_PARAM;
     }
 
     cb_name += sizeof(wchar_t);
 
-    cred = malloc(sizeof(kcdb_cred));
+    cred = PMALLOC(sizeof(kcdb_cred));
     ZeroMemory(cred, sizeof(kcdb_cred));
 
     cred->magic = KCDB_CRED_MAGIC;
     cred->identity = identity;
-    cred->name = malloc(cb_name);
+    cred->name = PMALLOC(cb_name);
     StringCbCopy(cred->name, cb_name, name);
     cred->type = cred_type;
 
@@ -107,9 +105,8 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_create(
     return KHM_ERROR_SUCCESS;
 }
 
-KHMEXP khm_int32 KHMAPI kcdb_cred_update(
-    khm_handle vdest,
-    khm_handle vsrc)
+KHMEXP khm_int32 KHMAPI kcdb_cred_update(khm_handle vdest,
+                                         khm_handle vsrc)
 {
     khm_int32 rv = KHM_ERROR_EQUIVALENT;
     kcdb_cred * src;
@@ -126,7 +123,7 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_update(
     kcdb_cred_lock_write();
 
     if(!kcdb_cred_is_active_cred(vsrc) ||
-        !kcdb_cred_is_active_cred(vdest))
+       !kcdb_cred_is_active_cred(vdest))
         goto _exit;
 
     src = (kcdb_cred *) vsrc;
@@ -159,10 +156,23 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_update(
             kcdb_buf_set_value(&dest->buf, i, i, srcbuf, cbsrcbuf);
             rv = KHM_ERROR_SUCCESS;
 
-_skip_copy:
+	_skip_copy:
             kcdb_attrib_release_info(a);
             kcdb_type_release_info(t);
-        }
+        } else {
+	    if (KHM_FAILED(kcdb_attrib_get_info(i, &a)))
+		continue;
+
+	    if (!(a->flags & KCDB_ATTR_FLAG_COMPUTED) &&
+		(a->flags & KCDB_ATTR_FLAG_TRANSIENT) &&
+		kcdb_cred_val_exist(dest, i)) {
+		kcdb_buf_set_value(&dest->buf, i, i, NULL, 0);
+
+		rv = KHM_ERROR_SUCCESS;
+	    }
+
+	    kcdb_attrib_release_info(a);
+	}
     }
 
     if (dest->flags != src->flags) {
@@ -177,7 +187,7 @@ _skip_copy:
             rv = KHM_ERROR_SUCCESS;
     }
 
-_exit:
+ _exit:
     kcdb_cred_unlock_write();
     return rv;
 }
@@ -192,24 +202,23 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_dup(
     khm_handle vnewcred;
 
     if(!pnewcred)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     *pnewcred = NULL;
 
     kcdb_cred_lock_write();
 
     if(!kcdb_cred_is_active_cred(vcred)) {
-        code = KHM_ERROR_INVALID_PARM;
+        code = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
     cred = (kcdb_cred *) vcred;
 
-    if(KHM_FAILED(kcdb_cred_create(
-        cred->name,
-        cred->identity,
-        cred->type,
-        &vnewcred))) 
+    if(KHM_FAILED(kcdb_cred_create(cred->name,
+                                   cred->identity,
+                                   cred->type,
+                                   &vnewcred))) 
     {
         code = KHM_ERROR_UNKNOWN;
         goto _exit;
@@ -236,20 +245,20 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_serial(
     kcdb_cred * c;
 
     if(!pserial)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
 
     if(!kcdb_cred_is_active_cred(vcred)) {
-        LockReleaseRead(&l_creds);
-        return KHM_ERROR_INVALID_PARM;
+        kcdb_cred_unlock_read();
+        return KHM_ERROR_INVALID_PARAM;
     }
 
     c = (kcdb_cred *) vcred;
 
     *pserial = c->id;
 
-    LockReleaseRead(&l_creds);
+    kcdb_cred_unlock_read();
 
     return KHM_ERROR_SUCCESS;
 }
@@ -261,12 +270,12 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_set_identity(
     kcdb_cred * c;
 
     if(!kcdb_is_identity(id))
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     kcdb_cred_lock_write();
     if(!kcdb_cred_is_active_cred(vcred)) {
         kcdb_cred_unlock_write();
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
     }
 
     c = (kcdb_cred *) vcred;
@@ -289,20 +298,20 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_type(
     kcdb_cred * c;
 
     if(!type)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
 
     if(!kcdb_cred_is_active_cred(vcred)) {
-        LockReleaseRead(&l_creds);
-        return KHM_ERROR_INVALID_PARM;
+        kcdb_cred_unlock_read();
+        return KHM_ERROR_INVALID_PARAM;
     }
 
     c = (kcdb_cred *) vcred;
 
     *type = c->type;
 
-    LockReleaseRead(&l_creds);
+    kcdb_cred_unlock_read();
 
     return KHM_ERROR_SUCCESS;
 }
@@ -316,7 +325,7 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_set_attrib(
     khm_int32 attr_id = -1;
 
     if(KHM_FAILED(kcdb_attrib_get_id(name, &attr_id)))
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     return kcdb_cred_set_attr(
         cred,
@@ -338,20 +347,20 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_set_attr(
     khm_int32 code = KHM_ERROR_SUCCESS;
 
     if(attr_id < 0 || attr_id > KCDB_ATTR_MAX_ID)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     kcdb_cred_lock_write();
 
     if(!kcdb_cred_is_active_cred(vcred)) {
         kcdb_cred_unlock_write();
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
     }
 
     cred = (kcdb_cred *) vcred;
 
     if(KHM_FAILED(kcdb_attrib_get_info(attr_id, &attrib))) {
         kcdb_cred_unlock_write();
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
     }
 
     if(attrib->flags & KCDB_ATTR_FLAG_COMPUTED)
@@ -371,7 +380,7 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_set_attr(
     if(KHM_FAILED(kcdb_type_get_info(attrib->type, &type))) {
         kcdb_cred_unlock_write();
         kcdb_attrib_release_info(attrib);
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
     }
 
     if(!(type->isValid(buffer,cbbuf))) {
@@ -380,7 +389,7 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_set_attr(
     }
 
     if((type->dup(buffer, cbbuf, NULL, &cbdest)) != KHM_ERROR_TOO_LONG) {
-        code = KHM_ERROR_INVALID_PARM;
+        code = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
@@ -450,12 +459,12 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_attrib_string(
         flags);
 }
 
-KHMEXP khm_int32 KHMAPI kcdb_cred_get_attr(
-    khm_handle vcred, 
-    khm_int32 attr_id,
-    khm_int32 * attr_type,
-    void * buffer, 
-    khm_size * pcbbuf)
+KHMEXP khm_int32 KHMAPI 
+kcdb_cred_get_attr(khm_handle vcred, 
+                   khm_int32 attr_id,
+                   khm_int32 * attr_type,
+                   void * buffer, 
+                   khm_size * pcbbuf)
 {
     khm_int32 code = KHM_ERROR_SUCCESS;
     kcdb_cred * cred = NULL;
@@ -463,10 +472,10 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_attr(
     kcdb_type * type = NULL;
 
     if(attr_id < 0 || attr_id > KCDB_ATTR_MAX_ID)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     if(KHM_FAILED(kcdb_attrib_get_info(attr_id, &attrib))) {
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
     }
 
     if(KHM_FAILED(kcdb_type_get_info(attrib->type, &type))) {
@@ -477,9 +486,9 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_attr(
     if(attr_type)
         *attr_type = attrib->type;
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
     if(!kcdb_cred_is_active_cred(vcred)) {
-        code = KHM_ERROR_INVALID_PARM;
+        code = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
@@ -511,7 +520,7 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_attr(
     }
 
 _exit:
-    LockReleaseRead(&l_creds);
+    kcdb_cred_unlock_read();
     if(type)
         kcdb_type_release_info(type);
     if(attrib)
@@ -533,29 +542,30 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_attr_string(
     kcdb_type * type = NULL;
 
     if(attr_id < 0 || attr_id > KCDB_ATTR_MAX_ID)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     if(KHM_FAILED(kcdb_attrib_get_info(attr_id, &attrib))) {
-        code = KHM_ERROR_INVALID_PARM;
-        goto _exit;
+        code = KHM_ERROR_INVALID_PARAM;
+        goto _exit_nolock;
     }
 
     if(KHM_FAILED(kcdb_type_get_info(attrib->type, &type))) {
         code = KHM_ERROR_UNKNOWN;
-        goto _exit;
+        goto _exit_nolock;
     }
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
     if(!kcdb_cred_is_active_cred(vcred)) {
-        code = KHM_ERROR_INVALID_PARM;
+        code = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
     cred = (kcdb_cred *) vcred;
 
     if(!buffer && !pcbbuf) {
-        /* in this case the caller is only trying to determine if the field
-            contains data.  We assume that computed fields are always non-null. */
+        /* in this case the caller is only trying to determine if the
+            field contains data.  We assume that computed fields are
+            always non-null. */
         code = (kcdb_cred_val_exist(cred, attr_id) ||
             (attrib->flags & KCDB_ATTR_FLAG_COMPUTED))?KHM_ERROR_SUCCESS:KHM_ERROR_NOT_FOUND;
         goto _exit;
@@ -565,27 +575,32 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_attr_string(
         void * buf;
         khm_size cbbuf;
 
-        code = attrib->compute_cb(
-            vcred,
-            attr_id,
-            NULL,
-            &cbbuf);
+        code = attrib->compute_cb(vcred,
+                                  attr_id,
+                                  NULL,
+                                  &cbbuf);
         if(code == KHM_ERROR_TOO_LONG) {
-            buf = malloc(cbbuf);
-            code = attrib->compute_cb(
-                vcred,
-                attr_id,
-                buf,
-                &cbbuf);
+            wchar_t vbuf[KCDB_MAXCCH_NAME];
+
+            if (cbbuf < sizeof(vbuf))
+                buf = vbuf;
+            else
+                buf = PMALLOC(cbbuf);
+
+            code = attrib->compute_cb(vcred,
+                                      attr_id,
+                                      buf,
+                                      &cbbuf);
             if(KHM_SUCCEEDED(code)) {
-                code = type->toString(
-                    buf,
-                    cbbuf,
-                    buffer,
-                    pcbbuf,
-                    flags);
+                code = type->toString(buf,
+                                      cbbuf,
+                                      buffer,
+                                      pcbbuf,
+                                      flags);
             }
-            free(buf);
+
+            if (buf != vbuf)
+                PFREE(buf);
         }
     } else {
         if(kcdb_cred_buf_exist(cred, attr_id)) {
@@ -599,8 +614,9 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_attr_string(
             code = KHM_ERROR_NOT_FOUND;
     }
 
-_exit:
-    LockReleaseRead(&l_creds);
+ _exit:
+    kcdb_cred_unlock_read();
+ _exit_nolock:
     if(type)
         kcdb_type_release_info(type);
     if(attrib)
@@ -620,12 +636,12 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_name(
     size_t cbsize;
 
     if(!cbbuf)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
     
     if(!kcdb_cred_is_active_cred(vcred)) {
-        code = KHM_ERROR_INVALID_PARM;
+        code = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
@@ -650,7 +666,7 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_name(
 
 _exit:
 
-    LockReleaseRead(&l_creds);
+    kcdb_cred_unlock_read();
     return code;
 }
 
@@ -662,12 +678,12 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_identity(
     kcdb_cred * cred;
 
     if(!identity)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
 
     if(!kcdb_cred_is_active_cred(vcred)) {
-        code = KHM_ERROR_INVALID_PARM;
+        code = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
@@ -678,7 +694,7 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_identity(
     *identity = cred->identity;
     
 _exit:
-    LockReleaseRead(&l_creds);
+    kcdb_cred_unlock_read();
     return code;
 }
 
@@ -689,8 +705,8 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_hold(khm_handle vcred)
 
     kcdb_cred_lock_write();
 
-    if(!kcdb_cred_is_active_cred(vcred)) {
-        code = KHM_ERROR_INVALID_PARM;
+    if(!kcdb_cred_is_cred(vcred)) {
+        code = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
@@ -710,8 +726,8 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_release(khm_handle vcred)
 
     kcdb_cred_lock_write();
 
-    if(!kcdb_cred_is_active_cred(vcred)) {
-        code = KHM_ERROR_INVALID_PARM;
+    if(!kcdb_cred_is_cred(vcred)) {
+        code = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
@@ -731,20 +747,17 @@ void kcdb_cred_check_and_delete(khm_handle vcred)
 {
     kcdb_cred * cred;
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
     if(!kcdb_cred_is_cred(vcred)) {
         goto _exit;
     }
 
     cred = (kcdb_cred *) vcred;
 
-    if(!(cred->flags & KCDB_CRED_FLAG_DELETED))
-        goto _exit;
-
     if(cred->refcount)
         goto _exit;
 
-    LockReleaseRead(&l_creds);
+    kcdb_cred_unlock_read();
     kcdb_cred_lock_write();
     if(!kcdb_cred_is_cred(vcred)) {
         /* did we lose the race? */
@@ -759,8 +772,8 @@ void kcdb_cred_check_and_delete(khm_handle vcred)
     LeaveCriticalSection(&cs_creds);
 
     kcdb_buf_delete(&cred->buf);
-    free(cred->name);
-    free(cred);
+    PFREE(cred->name);
+    PFREE(cred);
 
     /*TODO: notifications */
 
@@ -769,7 +782,7 @@ _exit2:
     return;
 
 _exit:
-    LockReleaseRead(&l_creds);
+    kcdb_cred_unlock_read();
 }
 
 KHMEXP khm_int32 KHMAPI kcdb_cred_delete(khm_handle vcred)
@@ -780,7 +793,7 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_delete(khm_handle vcred)
     kcdb_cred_lock_write();
 
     if(!kcdb_cred_is_active_cred(vcred)) {
-        code = KHM_ERROR_INVALID_PARM;
+        code = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
@@ -796,10 +809,10 @@ _exit:
     return code;
 }
 
-KHMEXP khm_int32 KHMAPI kcdb_creds_comp_attrib(
-    khm_handle cred1, 
-    khm_handle cred2, 
-    wchar_t * name)
+KHMEXP khm_int32 KHMAPI 
+kcdb_creds_comp_attrib(khm_handle cred1, 
+                       khm_handle cred2, 
+                       wchar_t * name)
 {
     khm_int32 attr_id;
 
@@ -809,10 +822,10 @@ KHMEXP khm_int32 KHMAPI kcdb_creds_comp_attrib(
     return kcdb_creds_comp_attr(cred1, cred2, attr_id);
 }
 
-KHMEXP khm_int32 KHMAPI kcdb_creds_comp_attr(
-    khm_handle vcred1, 
-    khm_handle vcred2, 
-    khm_int32 attr_id)
+KHMEXP khm_int32 KHMAPI 
+kcdb_creds_comp_attr(khm_handle vcred1, 
+                     khm_handle vcred2, 
+                     khm_int32 attr_id)
 {
     khm_int32 code = 0;
     kcdb_cred * cred1;
@@ -826,7 +839,7 @@ KHMEXP khm_int32 KHMAPI kcdb_creds_comp_attr(
     cred1 = (kcdb_cred *) vcred1;
     cred2 = (kcdb_cred *) vcred2;
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
     if(
         !kcdb_cred_is_active_cred(vcred1) ||
         !kcdb_cred_is_active_cred(vcred2))
@@ -858,6 +871,7 @@ KHMEXP khm_int32 KHMAPI kcdb_creds_comp_attr(
         goto _exit;
 
     if(attrib->flags & KCDB_ATTR_FLAG_COMPUTED) {
+        khm_octet  vbuf[KCDB_MAXCB_NAME * 2];
         void * buf1 = NULL;
         void * buf2 = NULL;
         khm_size cb1;
@@ -865,31 +879,43 @@ KHMEXP khm_int32 KHMAPI kcdb_creds_comp_attr(
 
         code = 0;
 
-        if(attrib->compute_cb(vcred1, attr_id, NULL, &cb1) != KHM_ERROR_TOO_LONG)
+        if(attrib->compute_cb(vcred1, attr_id, 
+                              NULL, &cb1) != KHM_ERROR_TOO_LONG)
             goto _exit_1;
 
-        if(attrib->compute_cb(vcred2, attr_id, NULL, &cb2) != KHM_ERROR_TOO_LONG)
+        if(attrib->compute_cb(vcred2, attr_id, 
+                              NULL, &cb2) != KHM_ERROR_TOO_LONG)
             goto _exit_1;
 
         if(cb1) {
-            buf1 = malloc(cb1);
+            if (cb1 < sizeof(vbuf))
+                buf1 = vbuf;
+            else
+                buf1 = PMALLOC(cb1);
+
             if(KHM_FAILED(attrib->compute_cb(vcred1, attr_id, buf1, &cb1)))
                 goto _exit_1;
         }
+
         if(cb2) {
-            buf2 = malloc(cb2);
+            if (cb1 + cb2 < sizeof(vbuf))
+                buf2 = vbuf + cb1;
+            else
+                buf2 = PMALLOC(cb2);
+
             if(KHM_FAILED(attrib->compute_cb(vcred2, attr_id, buf2, &cb2)))
                 goto _exit_1;
         }
-        code = type->comp(
-            buf1, cb1,
-            buf2, cb2);
-_exit_1:
-        if(buf1)
-            free(buf1);
-        if(buf2)
-            free(buf2);
 
+        code = type->comp(buf1, cb1,
+                          buf2, cb2);
+_exit_1:
+        if(buf1 && (buf1 < (void *)vbuf || 
+                    buf1 >= (void*)(vbuf + sizeof(vbuf))))
+            PFREE(buf1);
+        if(buf2 && (buf2 < (void *)vbuf ||
+                    buf2 >= (void *)(vbuf + sizeof(vbuf))))
+            PFREE(buf2);
     } else {
         code = type->comp(
             kcdb_cred_buf_get(cred1, attr_id),
@@ -899,7 +925,7 @@ _exit_1:
     }
 
 _exit:
-    LockReleaseRead(&l_creds);
+    kcdb_cred_unlock_read();
     if(attrib)
         kcdb_attrib_release_info(attrib);
     if(type)
@@ -907,41 +933,58 @@ _exit:
     return code;
 }
 
-KHMEXP khm_int32 KHMAPI kcdb_creds_is_equal(
-    khm_handle vcred1,
-    khm_handle vcred2)
+KHMEXP khm_int32 KHMAPI 
+kcdb_creds_is_equal(khm_handle vcred1,
+                    khm_handle vcred2)
 {
     khm_int32 code = 0;
     kcdb_cred * cred1;
     kcdb_cred * cred2;
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
     if(!kcdb_cred_is_active_cred(vcred1) ||
-        !kcdb_cred_is_active_cred(vcred2))
+       !kcdb_cred_is_active_cred(vcred2)) {
+
+        code = FALSE;
         goto _exit;
 
+    }
+
     if(vcred1 == vcred2) {
+
         code = TRUE;
         goto _exit;
+
     }
 
     cred1 = vcred1;
     cred2 = vcred2;
 
     if(cred1->identity == cred2->identity &&
-        cred1->type == cred2->type &&
-        !wcscmp(cred1->name, cred2->name)) {
+       cred1->type == cred2->type &&
+       !wcscmp(cred1->name, cred2->name)) {
+
+        kcdb_credtype * type;
+
         code = TRUE;
+
+        if (KHM_SUCCEEDED(kcdb_credtype_get_info(cred1->type, &type))) {
+            if (type->is_equal &&
+                (*type->is_equal)(vcred1, vcred2, NULL))
+                code = 0;
+
+            kcdb_credtype_release_info(type);
         }
+    }
 
 _exit:
-    LockReleaseRead(&l_creds);
+    kcdb_cred_unlock_read();
     return code;
 }
 
-KHMEXP khm_int32 KHMAPI kcdb_cred_get_flags(
-    khm_handle vcred,
-    khm_int32 * pflags)
+KHMEXP khm_int32 KHMAPI 
+kcdb_cred_get_flags(khm_handle vcred,
+                    khm_int32 * pflags)
 {
     khm_int32 f;
     khm_int32 rv = KHM_ERROR_SUCCESS;
@@ -949,12 +992,12 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_flags(
     int release_lock = TRUE;
 
     if (pflags == NULL)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
-    LockObtainRead(&l_creds);
+    kcdb_cred_lock_read();
     if (!kcdb_cred_is_active_cred(vcred)) {
         *pflags = 0;
-        rv = KHM_ERROR_INVALID_PARM;
+        rv = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
@@ -966,11 +1009,12 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_flags(
     if (!(f & KCDB_CRED_FLAG_EXPIRED) && 
         kcdb_cred_buf_exist(cred, KCDB_ATTR_EXPIRE)) {
 
-        khm_int64 ftc;
+        FILETIME ftc;
             
-        GetSystemTimeAsFileTime((LPFILETIME) &ftc);
-        if (ftc > *((khm_int64 *) 
-                    kcdb_cred_buf_get(cred, KCDB_ATTR_EXPIRE)))
+        GetSystemTimeAsFileTime(&ftc);
+        if (CompareFileTime(&ftc, ((FILETIME *) 
+                                   kcdb_cred_buf_get(cred, KCDB_ATTR_EXPIRE)))
+            >= 0)
             f |= KCDB_CRED_FLAG_EXPIRED;
     }
 
@@ -980,10 +1024,11 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_flags(
     if (!(f & KCDB_CRED_FLAG_INVALID)) {
         if (f & KCDB_CRED_FLAG_RENEWABLE) {
             if (kcdb_cred_buf_exist(cred, KCDB_ATTR_RENEW_EXPIRE)) {
-                khm_int64 ftc;
+                FILETIME ftc;
 
-                GetSystemTimeAsFileTime((LPFILETIME) &ftc);
-                if (ftc > *((khm_int64 *) kcdb_cred_buf_get(cred, KCDB_ATTR_RENEW_EXPIRE)))
+                GetSystemTimeAsFileTime(&ftc);
+                if (CompareFileTime(&ftc, ((FILETIME *)
+                                           kcdb_cred_buf_get(cred, KCDB_ATTR_RENEW_EXPIRE))) >= 0)
                     f |= KCDB_CRED_FLAG_INVALID;
             }
         } else {
@@ -995,16 +1040,16 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_flags(
     /* Commented out: this is a read operation.  We shouldn't attempt
        to lock for writing */
     if (f != cred->flags) {
-        LockReleaseRead(&l_creds);
-        LockObtainWrite(&l_creds);
+        kcdb_cred_unlock_read();
+        kcdb_cred_lock_write();
         /* Did we lose a race? */
         if (kcdb_cred_is_active_cred(vcred))
             cred->flags = f;
         else {
-            rv = KHM_ERROR_INVALID_PARM;
+            rv = KHM_ERROR_INVALID_PARAM;
             f = 0;
         }
-        LockReleaseWrite(&l_creds);
+        kcdb_cred_unlock_write();
         release_lock = FALSE;
     }
 #endif
@@ -1013,7 +1058,7 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_get_flags(
 
  _exit:
     if (release_lock)
-        LockReleaseRead(&l_creds);
+        kcdb_cred_unlock_read();
 
     return rv;
 }
@@ -1026,9 +1071,9 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_set_flags(
     khm_int32 rv = KHM_ERROR_SUCCESS;
     kcdb_cred * cred;
 
-    LockObtainWrite(&l_creds);
+    kcdb_cred_lock_write();
     if(!kcdb_cred_is_active_cred(vcred)) {
-        rv = KHM_ERROR_INVALID_PARM;
+        rv = KHM_ERROR_INVALID_PARAM;
         goto _exit;
     }
 
@@ -1042,6 +1087,6 @@ KHMEXP khm_int32 KHMAPI kcdb_cred_set_flags(
         (flags & mask);
 
  _exit:
-    LockReleaseWrite(&l_creds);
+    kcdb_cred_unlock_write();
     return rv;
 }
