@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Massachusetts Institute of Technology
+ * Copyright (c) 2005 Massachusetts Institute of Technology
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -46,6 +46,7 @@ static void
 add_subpanels(HWND hwnd, 
               khui_config_node ctx_node,
               khui_config_node ref_node) {
+
     HWND hw_tab;
     HWND hw_target;
     khui_config_node sub;
@@ -154,7 +155,7 @@ apply_all(HWND hwnd,
         assert(hw);
 #endif
 
-        PostMessage(hw, KHUI_WM_CFG_NOTIFY,
+        SendMessage(hw, KHUI_WM_CFG_NOTIFY,
                     MAKEWPARAM(0, WMCFG_APPLY), 0);
     }
 }
@@ -168,6 +169,7 @@ show_tab_panel(HWND hwnd,
     TCITEM tci;
     HWND hw;
     HWND hw_target;
+    HWND hw_firstctl;
     RECT r;
     RECT rref;
     khui_config_node_reg reg;
@@ -212,6 +214,11 @@ show_tab_panel(HWND hwnd,
                  r.right - r.left, r.bottom - r.top,
                  SWP_NOACTIVATE | SWP_NOOWNERZORDER |
                  SWP_SHOWWINDOW);
+
+    hw_firstctl = GetNextDlgTabItem(hw, NULL, FALSE);
+    if (hw_firstctl) {
+        SetFocus(hw_firstctl);
+    }
 }
 
 static INT_PTR
@@ -298,6 +305,12 @@ typedef struct tag_ident_data {
     HWND hwnd;
 } ident_data;
 
+typedef struct tag_global_props {
+    BOOL monitor;
+    BOOL auto_renew;
+    BOOL sticky;
+} global_props;
+
 typedef struct tag_idents_data {
     BOOL         valid;
 
@@ -305,6 +318,11 @@ typedef struct tag_idents_data {
     khm_size     n_idents;
     khm_size     nc_idents;
 
+    /* global options */
+    global_props saved;
+    global_props work;
+    BOOL         applied;
+
     int          refcount;
 
     HIMAGELIST   hi_status;
@@ -315,9 +333,15 @@ typedef struct tag_idents_data {
     int          idx_deleted;
 
     HWND         hwnd;
+    khui_config_init_data cfg;
 } idents_data;
 
-static idents_data cfg_idents = {FALSE, NULL, 0, 0, 0, NULL };
+static idents_data cfg_idents = {FALSE, NULL, 0, 0,
+                                 {0, 0, 0},
+                                 {0, 0, 0},
+                                 FALSE,
+
+                                 0, NULL };
 
 static void
 read_params_ident(ident_data * d) {
@@ -404,20 +428,36 @@ write_params_ident(ident_data * d) {
         return;
     }
 
-    if (d->saved.monitor != d->work.monitor)
-        khc_write_int32(csp_ident, L"Monitor", !!d->work.monitor);
+    if (d->removed) {
+        khm_handle h = NULL;
 
-    if (d->saved.auto_renew != d->work.auto_renew)
-        khc_write_int32(csp_ident, L"AllowAutoRenew", !!d->work.auto_renew);
+        khc_remove_space(csp_ident);
 
-    if (d->saved.sticky != d->work.sticky) {
-        khc_write_int32(csp_ident, L"Sticky", !!d->work.sticky);
-        if (d->work.sticky) {
-            kcdb_identity_set_flags(d->ident, KCDB_IDENT_FLAG_STICKY);
-        } else {
-            kcdb_identity_set_flags(d->ident, 
-                                    KCDB_IDENT_FLAG_STICKY |
-                                    KCDB_IDENT_FLAG_INVERT);
+        /* calling kcdb_identity_get_config() will update the
+           KCDB_IDENT_FLAG_CONFIG flag for the identity to reflect the
+           fact that it nolonger has a configuration. */
+        kcdb_identity_get_config(d->ident, 0, &h);
+        if (h) {
+            /* what the ? */
+#ifdef DEBUG
+            assert(FALSE);
+#endif
+            khc_close_space(h);
+        }
+
+    } else {
+
+        if (d->saved.monitor != d->work.monitor)
+            khc_write_int32(csp_ident, L"Monitor", !!d->work.monitor);
+
+        if (d->saved.auto_renew != d->work.auto_renew)
+            khc_write_int32(csp_ident, L"AllowAutoRenew",
+                            !!d->work.auto_renew);
+
+        if (d->saved.sticky != d->work.sticky) {
+            kcdb_identity_set_flags(d->ident,
+                                    (d->work.sticky)?KCDB_IDENT_FLAG_STICKY:0,
+                                    KCDB_IDENT_FLAG_STICKY);
         }
     }
 
@@ -435,6 +475,30 @@ write_params_ident(ident_data * d) {
 static void
 write_params_idents(void) {
     int i;
+    khm_handle csp_cw;
+
+    if (KHM_SUCCEEDED(khc_open_space(NULL, L"CredWindow",
+                                     KHM_FLAG_CREATE, &csp_cw))) {
+        if (cfg_idents.work.monitor != cfg_idents.saved.monitor) {
+            khc_write_int32(csp_cw, L"DefaultMonitor",
+                            !!cfg_idents.work.monitor);
+            cfg_idents.work.monitor = cfg_idents.saved.monitor;
+            cfg_idents.applied = TRUE;
+        }
+        if (cfg_idents.work.auto_renew != cfg_idents.saved.auto_renew) {
+            khc_write_int32(csp_cw, L"DefaultAllowAutoRenew",
+                            !!cfg_idents.work.auto_renew);
+            cfg_idents.work.auto_renew = cfg_idents.saved.auto_renew;
+            cfg_idents.applied = TRUE;
+        }
+        if (cfg_idents.work.sticky != cfg_idents.saved.sticky) {
+            khc_write_int32(csp_cw, L"DefaultMonitor",
+                            !!cfg_idents.work.sticky);
+            cfg_idents.work.sticky = cfg_idents.saved.sticky;
+            cfg_idents.applied = TRUE;
+        }
+        khc_close_space(csp_cw);
+    }
 
     for (i=0; i < (int)cfg_idents.n_idents; i++) {
         write_params_ident(&cfg_idents.idents[i]);
@@ -453,6 +517,7 @@ init_idents_data(void) {
     khm_size cb;
     int n_tries = 0;
     int i;
+    khm_handle csp_cw;
 
     if (cfg_idents.valid)
         return;
@@ -463,6 +528,35 @@ init_idents_data(void) {
     assert(cfg_idents.nc_idents == 0);
 #endif
 
+    if (KHM_SUCCEEDED(khc_open_space(NULL, L"CredWindow", 0, &csp_cw))) {
+        khm_int32 t;
+
+        if (KHM_SUCCEEDED(khc_read_int32(csp_cw, L"DefaultMonitor", &t)))
+            cfg_idents.saved.monitor = !!t;
+        else
+            cfg_idents.saved.monitor = TRUE;
+
+        if (KHM_SUCCEEDED(khc_read_int32(csp_cw, L"DefaultAllowAutoRenew", &t)))
+            cfg_idents.saved.auto_renew = !!t;
+        else
+            cfg_idents.saved.auto_renew = TRUE;
+
+        if (KHM_SUCCEEDED(khc_read_int32(csp_cw, L"DefaultSticky", &t)))
+            cfg_idents.saved.sticky = !!t;
+        else
+            cfg_idents.saved.sticky = FALSE;
+
+    } else {
+
+        cfg_idents.saved.monitor = TRUE;
+        cfg_idents.saved.auto_renew = TRUE;
+        cfg_idents.saved.sticky = FALSE;
+
+    }
+
+    cfg_idents.work = cfg_idents.saved;
+    cfg_idents.applied = FALSE;
+
     do {
         rv = kcdb_identity_enum(KCDB_IDENT_FLAG_CONFIG,
                                 KCDB_IDENT_FLAG_CONFIG,
@@ -476,8 +570,8 @@ init_idents_data(void) {
             break;
 
         if (widnames)
-            free(widnames);
-        widnames = malloc(cb);
+            PFREE(widnames);
+        widnames = PMALLOC(cb);
 #ifdef DEBUG
         assert(widnames);
 #endif
@@ -497,7 +591,7 @@ init_idents_data(void) {
         goto _cleanup;
     }
 
-    cfg_idents.idents = malloc(sizeof(*cfg_idents.idents) * 
+    cfg_idents.idents = PMALLOC(sizeof(*cfg_idents.idents) * 
                                cfg_idents.n_idents);
 #ifdef DEBUG
     assert(cfg_idents.idents);
@@ -518,7 +612,7 @@ init_idents_data(void) {
         StringCbLength(t, KCDB_IDENT_MAXCB_NAME, &cb);
         cb += sizeof(wchar_t);
 
-        cfg_idents.idents[i].idname = malloc(cb);
+        cfg_idents.idents[i].idname = PMALLOC(cb);
 #ifdef DEBUG
         assert(cfg_idents.idents[i].idname);
 #endif
@@ -540,7 +634,7 @@ init_idents_data(void) {
     cfg_idents.valid = TRUE;
 
     if (widnames)
-        free(widnames);
+        PFREE(widnames);
 }
 
 static void
@@ -554,11 +648,11 @@ free_idents_data(void) {
         if (cfg_idents.idents[i].ident)
             kcdb_identity_release(cfg_idents.idents[i].ident);
         if (cfg_idents.idents[i].idname)
-            free(cfg_idents.idents[i].idname);
+            PFREE(cfg_idents.idents[i].idname);
     }
 
     if (cfg_idents.idents)
-        free(cfg_idents.idents);
+        PFREE(cfg_idents.idents);
 
     cfg_idents.idents = NULL;
     cfg_idents.n_idents = 0;
@@ -587,252 +681,167 @@ release_idents_data(void) {
         free_idents_data();
 }
 
-#define BS_TRUE 1
-#define BS_FALSE 2
-
-static void
-refresh_view_idents_sel(HWND hwnd) {
-    HWND hw;
-    int sel_count;
-    int i;
-    int idx;
-    ident_data * d;
-    LVITEM lvi;
-
-    int monitor = 0;
-    int auto_renew = 0;
-    int sticky = 0;
-
-    hw = GetDlgItem(hwnd, IDC_CFG_IDENTS);
-
-    sel_count = ListView_GetSelectedCount(hw);
-
-    idx = -1;
-    for (i=0; i < sel_count; i++) {
-        idx = ListView_GetNextItem(hw, idx, LVNI_SELECTED);
-#ifdef DEBUG
-        assert(idx != -1);
-#endif
-        ZeroMemory(&lvi, sizeof(lvi));
-
-        lvi.iItem = idx;
-        lvi.iSubItem = 0;
-        lvi.mask = LVIF_PARAM;
-
-        ListView_GetItem(hw, &lvi);
-
-        d = (ident_data *) lvi.lParam;
-#ifdef DEBUG
-        assert(d != NULL);
-#endif
-
-        if (d->work.monitor)
-            monitor |= BS_TRUE;
-        else
-            monitor |= BS_FALSE;
-
-        if (d->work.auto_renew)
-            auto_renew |= BS_TRUE;
-        else
-            auto_renew |= BS_FALSE;
-
-        if (d->work.sticky)
-            sticky |= BS_TRUE;
-        else
-            sticky |= BS_FALSE;
-    }
-
-    CheckDlgButton(hwnd, IDC_CFG_MONITOR,
-                   (monitor == BS_TRUE)? BST_CHECKED:
-                   ((monitor == BS_FALSE)? BST_UNCHECKED:
-                    BST_INDETERMINATE));
-
-    CheckDlgButton(hwnd, IDC_CFG_RENEW,
-                   (auto_renew == BS_TRUE)? BST_CHECKED:
-                   ((auto_renew == BS_FALSE)? BST_UNCHECKED:
-                    BST_INDETERMINATE));
-
-    CheckDlgButton(hwnd, IDC_CFG_STICKY,
-                   (sticky == BS_TRUE)? BST_CHECKED:
-                   ((sticky == BS_FALSE)? BST_UNCHECKED:
-                    BST_INDETERMINATE));
-
-    if (sel_count > 0) {
-        EnableWindow(GetDlgItem(hwnd, IDC_CFG_REMOVE), TRUE);
-    } else {
-        EnableWindow(GetDlgItem(hwnd, IDC_CFG_REMOVE), FALSE);
-    }
-}
-
-#undef BS_TRUE
-#undef BS_FALSE
 
 static void
 refresh_data_idents(HWND hwnd) {
-    HWND hw;
-    int sel_count;
-    int i;
-    int idx;
-    ident_data * d;
-    LVITEM lvi;
-
-    UINT monitor = IsDlgButtonChecked(hwnd, IDC_CFG_MONITOR);
-    UINT auto_renew = IsDlgButtonChecked(hwnd, IDC_CFG_RENEW);
-    UINT sticky = IsDlgButtonChecked(hwnd, IDC_CFG_STICKY);
-
-    hw = GetDlgItem(hwnd, IDC_CFG_IDENTS);
-
-    sel_count = ListView_GetSelectedCount(hw);
-
-    idx = -1;
-    for (i=0; i < sel_count; i++) {
-        idx = ListView_GetNextItem(hw, idx, LVNI_SELECTED);
-#ifdef DEBUG
-        assert(idx != -1);
-#endif
-        ZeroMemory(&lvi, sizeof(lvi));
-
-        lvi.iItem = idx;
-        lvi.iSubItem = 0;
-        lvi.mask = LVIF_PARAM;
-
-        ListView_GetItem(hw, &lvi);
-
-        d = (ident_data *) lvi.lParam;
-#ifdef DEBUG
-        assert(d != NULL);
-#endif
-
-        if (monitor == BST_CHECKED)
-            d->work.monitor = TRUE;
-        else if (monitor == BST_UNCHECKED)
-            d->work.monitor = FALSE;
-
-        if (auto_renew == BST_CHECKED)
-            d->work.auto_renew = TRUE;
-        else if (auto_renew == BST_UNCHECKED)
-            d->work.auto_renew = FALSE;
-
-        if (sticky == BST_CHECKED)
-            d->work.sticky = TRUE;
-        else if (sticky == BST_UNCHECKED)
-            d->work.sticky = FALSE;
-
-        if (d->hwnd)
-            PostMessage(d->hwnd, KHUI_WM_CFG_NOTIFY,
-                        MAKEWPARAM(0, WMCFG_UPDATE_STATE), 0);
-    }
+    cfg_idents.work.monitor =
+        (IsDlgButtonChecked(hwnd, IDC_CFG_MONITOR) == BST_CHECKED);
+    cfg_idents.work.auto_renew =
+        (IsDlgButtonChecked(hwnd, IDC_CFG_RENEW) == BST_CHECKED);
+    cfg_idents.work.sticky =
+        (IsDlgButtonChecked(hwnd, IDC_CFG_STICKY) == BST_CHECKED);
 }
 
 static void
 refresh_view_idents_state(HWND hwnd) {
-    HWND hw;
-    int i;
-    LVITEM lvi;
-    ident_data * d;
+    BOOL modified;
+    BOOL applied;
+    khm_int32 flags = 0;
 
-    BOOL modified = FALSE;
-    BOOL applied = FALSE;
+    applied = cfg_idents.applied;
+    modified = (cfg_idents.work.monitor != cfg_idents.saved.monitor ||
+                cfg_idents.work.auto_renew != cfg_idents.saved.auto_renew ||
+                cfg_idents.work.sticky != cfg_idents.saved.sticky);
 
-    hw = GetDlgItem(hwnd, IDC_CFG_IDENTS);
+    if (modified)
+        flags |= KHUI_CNFLAG_MODIFIED;
+    if (applied)
+        flags |= KHUI_CNFLAG_APPLIED;
 
-    for (i = -1;;) {
+    khui_cfg_set_flags_inst(&cfg_idents.cfg, flags,
+                            KHUI_CNFLAG_APPLIED | KHUI_CNFLAG_MODIFIED);
+}
 
-        i = ListView_GetNextItem(hw, i, LVNI_ALL);
-        if (i == -1)
+/* dialog box procedure for the "Add new identity" dialog */
+INT_PTR CALLBACK
+khm_cfg_add_ident_proc(HWND hwnd,
+                       UINT umsg,
+                       WPARAM wParam,
+                       LPARAM lparam) {
+    switch(umsg) {
+    case WM_INITDIALOG:
+        /* set the max length of the edit control first */
+        SendDlgItemMessage(hwnd, IDC_CFG_IDNAME,
+                           EM_SETLIMITTEXT,
+                           KCDB_IDENT_MAXCCH_NAME - 1,
+                           0);
+        break;
+
+    case WM_DESTROY:
+        /* nor do we have to do anything here */
+        break;
+
+    case WM_COMMAND:
+        if (LOWORD(wParam) == IDOK) {
+            wchar_t idname[KCDB_IDENT_MAXCCH_NAME];
+            khm_int32 rv = KHM_ERROR_SUCCESS;
+            khm_handle ident = NULL;
+            khm_handle csp_ident = NULL;
+            khm_size i;
+            wchar_t err_msg[512] = L"";
+
+            GetDlgItemText(hwnd, IDC_CFG_IDNAME, idname,
+                           ARRAYLENGTH(idname));
+
+            idname[ARRAYLENGTH(idname) - 1] = L'\0';
+            if (KHM_FAILED(rv = kcdb_identpro_validate_name(idname)) &&
+                rv != KHM_ERROR_NO_PROVIDER &&
+                rv != KHM_ERROR_NOT_IMPLEMENTED) {
+                /* the supplied name was invalid or something */
+
+                wchar_t fmt[256];
+
+                LoadString(khm_hInstance, IDS_CFG_IDNAME_INV,
+                           fmt, ARRAYLENGTH(fmt));
+                StringCbPrintf(err_msg, sizeof(err_msg), fmt, idname);
+
+                goto show_failure;
+            }
+
+            /* now check if this is actually a new identity */
+            for (i=0; i < cfg_idents.n_idents; i++) {
+                if (!kcdb_identpro_compare_name(cfg_idents.idents[i].idname,
+                                                idname))
+                    break;
+            }
+
+            if (i < cfg_idents.n_idents) {
+                wchar_t fmt[256];
+
+                LoadString(khm_hInstance, IDS_CFG_IDNAME_EXT,
+                           fmt, ARRAYLENGTH(fmt));
+                StringCbPrintf(err_msg, sizeof(err_msg), fmt, idname);
+
+                goto show_failure;
+            }
+
+            /* ok.  now we are all set to add the new identity */
+            if (KHM_FAILED(rv = kcdb_identity_create(idname,
+                                                     KCDB_IDENT_FLAG_CREATE,
+                                                     &ident))) {
+                /* oops */
+                wchar_t fmt[256];
+
+                LoadString(khm_hInstance, IDS_CFG_IDNAME_CCR,
+                           fmt, ARRAYLENGTH(fmt));
+                StringCbPrintf(err_msg, sizeof(err_msg), fmt, rv);
+
+                goto show_failure;
+            }
+
+            /* now we have to create the identity configuration. */
+            if (KHM_FAILED(rv = kcdb_identity_get_config(ident,
+                                                         KHM_FLAG_CREATE,
+                                                         &csp_ident))) {
+                wchar_t fmt[256];
+
+                LoadString(khm_hInstance, IDS_CFG_IDNAME_CCC,
+                           fmt, ARRAYLENGTH(fmt));
+                StringCbPrintf(err_msg, sizeof(err_msg), fmt, rv);
+
+                kcdb_identity_release(ident);
+
+                goto show_failure;
+            }
+
+            khm_refresh_config();
+
+            kcdb_identity_release(ident);
+            khc_close_space(csp_ident);
+
+            EndDialog(hwnd, 0);
             break;
 
-        ZeroMemory(&lvi, sizeof(lvi));
-        lvi.iItem = i;
-        lvi.iSubItem = 0;
-        lvi.mask = LVIF_PARAM;
+        show_failure:
+            {
+                wchar_t title[512];
+                wchar_t fmt[256];
 
-        ListView_GetItem(hw, &lvi);
+                if (!err_msg[0])
+                    break;
 
-        d = (ident_data *) lvi.lParam;
-#ifdef DEBUG
-        assert(d != NULL);
-#endif
+                LoadString(khm_hInstance, IDS_CFG_IDNAME_PRB,
+                           fmt, ARRAYLENGTH(fmt));
+                StringCbPrintf(title, sizeof(title), fmt, idname);
 
-        ZeroMemory(&lvi, sizeof(lvi));
+                MessageBox(hwnd, err_msg, title, MB_OK | MB_ICONSTOP);
 
-        lvi.mask = LVIF_STATE;
-        lvi.stateMask = LVIS_STATEIMAGEMASK;
-        lvi.iItem = i;
-        lvi.iSubItem = 0;
-
-        if (d->removed) {
-            lvi.state = INDEXTOSTATEIMAGEMASK(cfg_idents.idx_deleted);
-            modified = TRUE;
-        } else if (d->saved.monitor != d->work.monitor ||
-                   d->saved.auto_renew != d->work.auto_renew ||
-                   d->saved.sticky != d->work.sticky) {
-            lvi.state = INDEXTOSTATEIMAGEMASK(cfg_idents.idx_modified);
-            modified = TRUE;
-        } else if (d->applied) {
-            lvi.state = INDEXTOSTATEIMAGEMASK(cfg_idents.idx_applied);
-            applied = TRUE;
-        } else {
-            lvi.state = INDEXTOSTATEIMAGEMASK(cfg_idents.idx_default);
+                /* don't end the dialog yet */
+                break;
+            }
+            break;
+            
+        } else if (LOWORD(wParam) == IDCANCEL) {
+            EndDialog(hwnd, 1);
         }
-
-        ListView_SetItem(hw, &lvi);
+        break;
     }
 
-    {
-        khm_int32 flags = 0;
-        khui_config_node node = NULL;
-
-        if (modified)
-            flags |= KHUI_CNFLAG_MODIFIED;
-        if (applied)
-            flags |= KHUI_CNFLAG_APPLIED;
-
-        khui_cfg_open(NULL, L"KhmIdentities", &node);
-#ifdef DEBUG
-        assert(node);
-#endif
-        khui_cfg_set_flags(node, flags,
-                           KHUI_CNFLAG_APPLIED | KHUI_CNFLAG_MODIFIED);
-    }
-}
-
-static void
-remove_idents(HWND hwnd) {
-    HWND hw;
-    int sel_count;
-    int i;
-    int idx;
-    ident_data * d;
-    LVITEM lvi;
-
-    hw = GetDlgItem(hwnd, IDC_CFG_IDENTS);
-
-    sel_count = ListView_GetSelectedCount(hw);
-
-    idx = -1;
-    for (i=0; i < sel_count; i++) {
-        idx = ListView_GetNextItem(hw, idx, LVNI_SELECTED);
-#ifdef DEBUG
-        assert(idx != -1);
-#endif
-        ZeroMemory(&lvi, sizeof(lvi));
-
-        lvi.iItem = idx;
-        lvi.iSubItem = 0;
-        lvi.mask = LVIF_PARAM;
-
-        ListView_GetItem(hw, &lvi);
-
-        d = (ident_data *) lvi.lParam;
-#ifdef DEBUG
-        assert(d != NULL);
-#endif
-
-        d->removed = TRUE;
-    }
+    return FALSE;
 }
 
+/* dialog procedure for the "general" pane of the "identities"
+   configuration node. */
 INT_PTR CALLBACK
 khm_cfg_ids_tab_proc(HWND hwnd,
                     UINT umsg,
@@ -842,45 +851,28 @@ khm_cfg_ids_tab_proc(HWND hwnd,
     switch(umsg) {
     case WM_INITDIALOG:
         {
-            HWND hw;
             HICON hicon;
-            LVCOLUMN lvcol;
-            LVITEM lvi;
-            wchar_t coltext[256];
-            RECT r;
-            int i;
 
             hold_idents_data();
 
             cfg_idents.hwnd = hwnd;
+            cfg_idents.cfg = *((khui_config_init_data *) lParam);
 
-            /* first add the column */
-            hw = GetDlgItem(hwnd, IDC_CFG_IDENTS);
-
-            ZeroMemory(&lvcol, sizeof(lvcol));
-            lvcol.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT;
-
-            lvcol.fmt = LVCFMT_IMAGE | LVCFMT_LEFT;
-
-            GetWindowRect(hw, &r);
-            lvcol.cx = ((r.right - r.left) * 95) / 100;
-
-            LoadString(khm_hInstance, IDS_CFG_IDS_IDENTITY,
-                       coltext, ARRAYLENGTH(coltext));
-            lvcol.pszText = coltext;
-
-            ListView_InsertColumn(hw, 0, &lvcol);
-
-            /* and the status icons */
+            /* add the status icons */
             if (cfg_idents.hi_status)
                 goto _done_with_icons;
 
-            cfg_idents.hi_status = ImageList_Create(SM_CXICON, SM_CYICON, 
-                                                    ILC_COLOR8 | ILC_MASK,
-                                                    4,4);
+            cfg_idents.hi_status = 
+                ImageList_Create(GetSystemMetrics(SM_CXSMICON),
+                                 GetSystemMetrics(SM_CYSMICON), 
+                                 ILC_COLOR8 | ILC_MASK,
+                                 4,4);
 
-            hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_ID),
-                              IMAGE_ICON, SM_CXICON, SM_CYICON, LR_DEFAULTCOLOR);
+            hicon =
+                LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_ID),
+                          IMAGE_ICON,
+                          GetSystemMetrics(SM_CXSMICON),
+                          GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
             cfg_idents.idx_id = ImageList_AddIcon(cfg_idents.hi_status,
                                                   hicon);
@@ -888,7 +880,8 @@ khm_cfg_ids_tab_proc(HWND hwnd,
             DestroyIcon(hicon);
 
             hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_DEFAULT),
-                              IMAGE_ICON, SM_CXICON, SM_CYICON, LR_DEFAULTCOLOR);
+                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), 
+                              GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
             cfg_idents.idx_default = ImageList_AddIcon(cfg_idents.hi_status, 
                                                        hicon) + 1;
@@ -896,7 +889,8 @@ khm_cfg_ids_tab_proc(HWND hwnd,
             DestroyIcon(hicon);
 
             hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_MODIFIED),
-                              IMAGE_ICON, SM_CXICON, SM_CYICON, LR_DEFAULTCOLOR);
+                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), 
+                              GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
             cfg_idents.idx_modified = ImageList_AddIcon(cfg_idents.hi_status, 
                                                         hicon) + 1;
@@ -904,7 +898,8 @@ khm_cfg_ids_tab_proc(HWND hwnd,
             DestroyIcon(hicon);
 
             hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_APPLIED),
-                              IMAGE_ICON, SM_CXICON, SM_CYICON, LR_DEFAULTCOLOR);
+                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), 
+                              GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
             cfg_idents.idx_applied = ImageList_AddIcon(cfg_idents.hi_status, 
                                                        hicon) + 1;
@@ -912,63 +907,44 @@ khm_cfg_ids_tab_proc(HWND hwnd,
             DestroyIcon(hicon);
 
             hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_DELETED),
-                              IMAGE_ICON, SM_CXICON, SM_CYICON, LR_DEFAULTCOLOR);
+                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), 
+                              GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
             cfg_idents.idx_deleted = ImageList_AddIcon(cfg_idents.hi_status, 
                                                        hicon) + 1;
 
             DestroyIcon(hicon);
 
-            ListView_SetImageList(hw, cfg_idents.hi_status, LVSIL_SMALL);
-            ListView_SetImageList(hw, cfg_idents.hi_status, LVSIL_STATE);
-
         _done_with_icons:
 
-            /* now add each identity */
-            for(i=0; i < (int)cfg_idents.n_idents; i++) {
-                ZeroMemory(&lvi, sizeof(lvi));
+            CheckDlgButton(hwnd, IDC_CFG_MONITOR,
+                           (cfg_idents.work.monitor)?BST_CHECKED:BST_UNCHECKED);
+            CheckDlgButton(hwnd, IDC_CFG_RENEW,
+                           (cfg_idents.work.auto_renew)?BST_CHECKED:BST_UNCHECKED);
+            CheckDlgButton(hwnd, IDC_CFG_STICKY,
+                           (cfg_idents.work.sticky)?BST_CHECKED:BST_UNCHECKED);
 
-                lvi.mask = LVIF_PARAM | LVIF_TEXT | LVIF_STATE | LVIF_IMAGE;
-                lvi.iImage = cfg_idents.idx_id;
-                lvi.lParam = (LPARAM) &cfg_idents.idents[i];
-                lvi.pszText = cfg_idents.idents[i].idname;
-                lvi.state = INDEXTOSTATEIMAGEMASK(cfg_idents.idx_default);
-                lvi.stateMask = LVIS_STATEIMAGEMASK;
-
-                cfg_idents.idents[i].lv_idx = ListView_InsertItem(hw, &lvi);
-            }
-
-            ListView_SetView(hw, LV_VIEW_DETAILS);
         }
         return FALSE;
 
-    case WM_NOTIFY:
-        {
-            LPNMHDR lpnm = (LPNMHDR) lParam;
-
-            if (lpnm->code == LVN_ITEMCHANGED) {
-                refresh_view_idents_sel(hwnd);
-            }
-        }
-        return TRUE;
-
     case WM_COMMAND:
 
         if (HIWORD(wParam) == BN_CLICKED) {
             UINT ctrl = LOWORD(wParam);
+
             switch(ctrl) {
             case IDC_CFG_MONITOR:
             case IDC_CFG_RENEW:
             case IDC_CFG_STICKY:
-                if (IsDlgButtonChecked(hwnd, ctrl) == BST_CHECKED)
-                    CheckDlgButton(hwnd, ctrl, BST_UNCHECKED);
-                else
-                    CheckDlgButton(hwnd, ctrl, BST_CHECKED);
                 refresh_data_idents(hwnd);
                 break;
 
-            case IDC_CFG_REMOVE:
-                remove_idents(hwnd);
+            case IDC_CFG_ADDIDENT:
+                DialogBoxParam(khm_hInstance,
+                               MAKEINTRESOURCE(IDD_CFG_ADDIDENT),
+                               hwnd,
+                               khm_cfg_add_ident_proc,
+                               (LPARAM) hwnd);
                 break;
             }
 
@@ -987,7 +963,6 @@ khm_cfg_ids_tab_proc(HWND hwnd,
 
             case WMCFG_UPDATE_STATE:
                 refresh_view_idents_state(hwnd);
-                refresh_view_idents_sel(hwnd);
                 break;
             }
         }
@@ -1010,6 +985,7 @@ khm_cfg_ids_tab_proc(HWND hwnd,
     return FALSE;
 }
 
+/* dialog procedure for the "Identities" configuration node */
 INT_PTR CALLBACK
 khm_cfg_identities_proc(HWND hwnd,
                         UINT uMsg,
@@ -1079,6 +1055,26 @@ refresh_view_ident(HWND hwnd, khui_config_node node) {
                    (d->work.sticky? BST_CHECKED: BST_UNCHECKED));
 }
 
+static void
+mark_remove_ident(HWND hwnd, khui_config_init_data * idata) {
+    ident_data * d;
+
+    d = find_ident_by_node(idata->ctx_node);
+#ifdef DEBUG
+    assert(d);
+#endif
+
+    if (d->removed)
+        return;
+
+    d->removed = TRUE;
+
+    khui_cfg_set_flags_inst(idata, KHUI_CNFLAG_MODIFIED,
+                            KHUI_CNFLAG_MODIFIED);
+
+    EnableWindow(GetDlgItem(hwnd, IDC_CFG_REMOVE), FALSE);
+}
+
 static void
 refresh_data_ident(HWND hwnd, khui_config_init_data * idata) {
     ident_data * d;
@@ -1116,6 +1112,8 @@ refresh_data_ident(HWND hwnd, khui_config_init_data * idata) {
     }
 }
 
+/* dialog procedure for the "general" pane of individual identity
+   configuration nodes. */
 INT_PTR CALLBACK
 khm_cfg_id_tab_proc(HWND hwnd,
                     UINT umsg,
@@ -1161,8 +1159,16 @@ khm_cfg_id_tab_proc(HWND hwnd,
                     PostMessage(cfg_idents.hwnd, KHUI_WM_CFG_NOTIFY,
                                 MAKEWPARAM(1, WMCFG_UPDATE_STATE), 0);
                 break;
+
+            case IDC_CFG_REMOVE:
+                mark_remove_ident(hwnd, idata);
+                if (cfg_idents.hwnd)
+                    PostMessage(cfg_idents.hwnd, KHUI_WM_CFG_NOTIFY,
+                                MAKEWPARAM(1, WMCFG_UPDATE_STATE), 0);
+                break;
             }
         }
+
         khm_set_dialog_result(hwnd, 0);
         return TRUE;
 
@@ -1209,6 +1215,7 @@ khm_cfg_id_tab_proc(HWND hwnd,
     return FALSE;
 }
 
+/* dialog procedure for individual identity configuration nodes */
 INT_PTR CALLBACK
 khm_cfg_identity_proc(HWND hwnd,
                       UINT uMsg,
