@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Massachusetts Institute of Technology
+ * Copyright (c) 2005 Massachusetts Institute of Technology
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -145,12 +145,12 @@ cfgui_initialize_dialog(HWND hwnd) {
 
     /* create and fill the image list for the treeview */
 
-    d->hi_status = ImageList_Create(SM_CXICON, SM_CYICON, 
+    d->hi_status = ImageList_Create(GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), 
                                     ILC_COLOR8 | ILC_MASK,
                                     4,4);
 
     hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_DEFAULT),
-                      IMAGE_ICON, SM_CXICON, SM_CYICON, LR_DEFAULTCOLOR);
+                      IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
     /* note that we can't use index 0 because that is used to indicate
        that there is no state image for the node */
@@ -161,14 +161,14 @@ cfgui_initialize_dialog(HWND hwnd) {
     DestroyIcon(hicon);
 
     hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_MODIFIED),
-                      IMAGE_ICON, SM_CXICON, SM_CYICON, LR_DEFAULTCOLOR);
+                      IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
     d->idx_modified = ImageList_AddIcon(d->hi_status, hicon);
 
     DestroyIcon(hicon);
 
     hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_APPLIED),
-                      IMAGE_ICON, SM_CXICON, SM_CYICON, LR_DEFAULTCOLOR);
+                      IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
     d->idx_applied = ImageList_AddIcon(d->hi_status, hicon);
 
@@ -253,6 +253,42 @@ cfgui_uninitialize_dialog(HWND hwnd) {
         ImageList_Destroy(d->hi_status);
 }
 
+static HWND
+cfgui_create_config_node_window(HWND hwnd, khui_config_node node) {
+    khui_config_node_reg reg;
+    khm_int32 rv;
+    HWND hw_new;
+
+    khui_config_node parent;
+
+    if (KHM_SUCCEEDED(khui_cfg_get_parent(node, &parent))) {
+        HWND hwp;
+
+        hwp = khui_cfg_get_hwnd(parent);
+
+        if (hwp == NULL)
+            cfgui_create_config_node_window(hwnd, parent);
+
+        khui_cfg_release(parent);
+    }
+
+    rv = khui_cfg_get_reg(node, &reg);
+#ifdef DEBUG
+    assert(KHM_SUCCEEDED(rv));
+#endif
+    hw_new = CreateDialogParam(reg.h_module,
+                               reg.dlg_template,
+                               hwnd,
+                               reg.dlg_proc,
+                               (LPARAM) node);
+#ifdef DEBUG
+    assert(hw_new);
+#endif
+    khui_cfg_set_hwnd(node, hw_new);
+
+    return hw_new;
+}
+
 static void
 cfgui_activate_node(HWND hwnd, khui_config_node node) {
 
@@ -273,25 +309,11 @@ cfgui_activate_node(HWND hwnd, khui_config_node node) {
     if (node == NULL) {
         hw_new = d->hw_generic_pane;
     } else {
-        khui_config_node_reg reg;
-        khm_int32 rv;
 
         hw_new = khui_cfg_get_hwnd(node);
 
         if (hw_new == NULL) {
-            rv = khui_cfg_get_reg(node, &reg);
-#ifdef DEBUG
-            assert(KHM_SUCCEEDED(rv));
-#endif
-            hw_new = CreateDialogParam(reg.h_module,
-                                       reg.dlg_template,
-                                       hwnd,
-                                       reg.dlg_proc,
-                                       (LPARAM) node);
-#ifdef DEBUG
-            assert(hw_new);
-#endif
-            khui_cfg_set_hwnd(node, hw_new);
+            hw_new = cfgui_create_config_node_window(hwnd, node);
         }
     }
 
@@ -380,13 +402,16 @@ static void
 cfgui_apply_settings(khui_config_node node) {
     HWND hwnd;
     khui_config_node c;
+    khm_int32 flags;
 
     hwnd = khui_cfg_get_hwnd(node);
+    flags = khui_cfg_get_flags(node);
 
-    if (hwnd)
+    if (hwnd && (flags & KHUI_CNFLAG_MODIFIED)) {
         SendMessage(hwnd, KHUI_WM_CFG_NOTIFY,
                     MAKEWPARAM(0, WMCFG_APPLY),
                     (LPARAM) node);
+    }
 
     if (KHM_FAILED(khui_cfg_get_first_child(node, &c)))
         return;
@@ -428,10 +453,8 @@ cfgui_update_state(HWND hwnd,
     TreeView_SetItem(hwtv, &itx);
 
     if(cfgui_check_mod_state(NULL)) {
-        EnableWindow(GetDlgItem(hwnd, IDC_CFG_SUMMARY), TRUE);
         EnableWindow(GetDlgItem(hwnd, IDAPPLY), TRUE);
     } else {
-        EnableWindow(GetDlgItem(hwnd, IDC_CFG_SUMMARY), FALSE);
         EnableWindow(GetDlgItem(hwnd, IDAPPLY), FALSE);
     }
 }
@@ -490,7 +513,7 @@ cfgui_dlgproc_generic(HWND hwnd,
             r_fill.bottom = r_logo.top;
             FillRect(hdc, &r_fill, d->hbr_white);
 
-            SetWindowLong(hwnd, DWL_MSGRESULT, (LONG) TRUE);
+            SetWindowLongPtr(hwnd, DWLP_MSGRESULT, (LONG) TRUE);
         }
         return TRUE;
     }
@@ -513,7 +536,9 @@ cfgui_dlgproc(HWND hwnd,
 
         khui_cfg_clear_params();
 
-        d = malloc(sizeof(*d));
+        khui_cfg_set_configui_handle(hwnd);
+
+        d = PMALLOC(sizeof(*d));
         ZeroMemory(d, sizeof(*d));
 
         d->hbr_white = CreateSolidBrush(RGB(255,255,255));
@@ -543,8 +568,6 @@ cfgui_dlgproc(HWND hwnd,
         khm_add_dialog(hwnd);
         khm_enter_modal(hwnd);
 
-        khui_cfg_set_configui_handle(hwnd);
-
         return TRUE;
 
     case WM_DESTROY:
@@ -558,7 +581,6 @@ cfgui_dlgproc(HWND hwnd,
         khui_delete_bitmap(&d->kbmp_logo);
         DeleteObject(d->hbr_white);
 
-        khm_leave_modal();
         khm_del_dialog(hwnd);
 
         SetForegroundWindow(khm_hwnd_main);
@@ -569,6 +591,8 @@ cfgui_dlgproc(HWND hwnd,
         {
             LPNMHDR lpnm;
             LPNMTREEVIEW lptv;
+            LPNMTVGETINFOTIP lpgi;
+            khui_config_node node;
 
             lpnm = (LPNMHDR) lParam;
 
@@ -579,6 +603,31 @@ cfgui_dlgproc(HWND hwnd,
                                     (khui_config_node) 
                                     lptv->itemNew.lParam);
                 return TRUE;
+
+            case TVN_GETINFOTIP:
+                lpgi = (LPNMTVGETINFOTIP) lParam;
+                node = (khui_config_node) lpgi->lParam;
+
+                if (node) {
+                    khm_int32 flags = 0;
+
+                    flags = khui_cfg_get_flags(node);
+
+                    if (flags & KHUI_CNFLAG_MODIFIED) {
+                        LoadString(khm_hInstance, IDS_CFG_IT_MOD,
+                                   lpgi->pszText, lpgi->cchTextMax);
+                    } else if (flags & KHUI_CNFLAG_APPLIED) {
+                        LoadString(khm_hInstance, IDS_CFG_IT_APP,
+                                   lpgi->pszText, lpgi->cchTextMax);
+                    } else {
+                        LoadString(khm_hInstance, IDS_CFG_IT_NONE,
+                                   lpgi->pszText, lpgi->cchTextMax);
+                    }
+                } else {
+                    StringCchCopy(lpgi->pszText, lpgi->cchTextMax, L"");
+                }
+
+                return TRUE;
             }
         }
         return TRUE;
@@ -593,6 +642,7 @@ cfgui_dlgproc(HWND hwnd,
     case WM_COMMAND:
         switch(wParam) {
         case MAKEWPARAM(IDCANCEL, BN_CLICKED):
+            khm_leave_modal();
             DestroyWindow(hwnd);
             break;
 
@@ -602,6 +652,7 @@ cfgui_dlgproc(HWND hwnd,
 
         case MAKEWPARAM(IDOK, BN_CLICKED):
             cfgui_apply_settings(NULL);
+            khm_leave_modal();
             DestroyWindow(hwnd);
             break;
         }
@@ -617,7 +668,12 @@ cfgui_dlgproc(HWND hwnd,
             cfgui_update_state(hwnd, LOWORD(wParam), 
                                (khui_config_node) lParam);
             break;
+
+        case WMCFG_SYNC_NODE_LIST:
+            /*TODO: synchronize the node lists here */
+            break;
         }
+
         return TRUE;
     }
 
@@ -669,6 +725,10 @@ void khm_refresh_config(void) {
     khm_int32 rv;
     int n_tries = 0;
     khui_config_node cfg_ids = NULL;
+    khui_config_node cfg_r = NULL;
+    khui_config_node cfg_iter = NULL;
+    khui_menu_def * omenu;
+    khm_boolean refresh_menu = FALSE;
 
     do {
         rv = kcdb_identity_enum(KCDB_IDENT_FLAG_CONFIG,
@@ -682,8 +742,8 @@ void khm_refresh_config(void) {
             return;
 
         if (idents)
-            free(idents);
-        idents = malloc(cb);
+            PFREE(idents);
+        idents = PMALLOC(cb);
 #ifdef DEBUG
         assert(idents);
 #endif
@@ -744,12 +804,106 @@ void khm_refresh_config(void) {
         }
     }
 
+    for (khui_cfg_get_first_child(cfg_ids, &cfg_iter);
+         cfg_iter;
+         khui_cfg_get_next_release(&cfg_iter)) {
+
+        wchar_t cfgname[KCDB_IDENT_MAXCCH_NAME];
+        khm_size cb;
+        khm_handle tident = NULL;
+        khm_int32 tflags = 0;
+
+        cb = sizeof(cfgname);
+        khui_cfg_get_name(cfg_iter, cfgname, &cb);
+
+        if (KHM_FAILED(kcdb_identity_create(cfgname, 0, &tident)) ||
+            KHM_FAILED(kcdb_identity_get_flags(tident, &tflags)) ||
+            !(tflags & KCDB_IDENT_FLAG_ACTIVE) ||
+            !(tflags & KCDB_IDENT_FLAG_CONFIG)) {
+
+            /* this configuration node needs to be removed */
+
+            khui_cfg_remove(cfg_iter);
+        }
+    }
+
+    /* Now iterate through the root level configuration nodes and make
+       sure we have a menu item for each of them. */
+    if (KHM_FAILED(khui_cfg_get_first_child(NULL, &cfg_r)))
+        goto _cleanup;
+
+    omenu = khui_find_menu(KHUI_MENU_OPTIONS);
+    if (omenu == NULL)
+        goto _cleanup;
+
+    do {
+        khm_int32 action;
+        khm_int32 flags;
+        khui_action * paction;
+        wchar_t cname[KHUI_MAXCCH_NAME];
+        wchar_t wshort[KHUI_MAXCCH_SHORT_DESC];
+        khm_size cb;
+        khm_handle sub;
+        khui_config_node_reg reg;
+
+        flags = khui_cfg_get_flags(cfg_r);
+        if (flags & KHUI_CNFLAG_SYSTEM)
+            goto _next_cfg;
+
+        cb = sizeof(cname);
+        if (KHM_FAILED(khui_cfg_get_name(cfg_r, cname, &cb))) {
+#ifdef DEBUG
+            assert(FALSE);
+#endif
+            goto _next_cfg;
+        }
+
+        paction = khui_find_named_action(cname);
+
+        if (!paction) {
+            khui_cfg_get_reg(cfg_r, &reg);
+
+            kmq_create_hwnd_subscription(khm_hwnd_main, &sub);
+
+            StringCbCopy(wshort, sizeof(wshort), reg.short_desc);
+            StringCbCat(wshort, sizeof(wshort), L" ...");
+
+            action = khui_action_create(cname,
+                                        wshort,
+                                        reg.long_desc,
+                                        (void *) CFGACTION_MAGIC,
+                                        KHUI_ACTIONTYPE_TRIGGER,
+                                        sub);
+
+            if (action == 0) {
+#ifdef DEBUG
+                assert(FALSE);
+#endif
+                goto _next_cfg;
+            }
+
+            khui_menu_insert_action(omenu, -1, action, 0);
+
+            refresh_menu = TRUE;
+        }
+
+    _next_cfg:
+        if (KHM_FAILED(khui_cfg_get_next_release(&cfg_r)))
+            break;
+    } while(cfg_r);
+
+    if (refresh_menu)
+        khm_menu_refresh_items();
+
  _cleanup:
     if (cfg_ids)
         khui_cfg_release(cfg_ids);
 
+    if (cfg_r)
+        khui_cfg_release(cfg_r);
+
     if (idents)
-        free(idents);
+        PFREE(idents);
 }
 
 void khm_init_config(void) {
@@ -761,7 +915,7 @@ void khm_init_config(void) {
     reg.short_desc = wshort;
     reg.long_desc = wlong;
     reg.h_module = khm_hInstance;
-    reg.flags = 0;
+    reg.flags = KHUI_CNFLAG_SYSTEM;
 
     reg.name = L"KhmGeneral";
     reg.dlg_template = MAKEINTRESOURCE(IDD_CFG_GENERAL);
@@ -773,6 +927,16 @@ void khm_init_config(void) {
 
     khui_cfg_register(NULL, &reg);
 
+    reg.name = L"KhmAppear";
+    reg.dlg_template = MAKEINTRESOURCE(IDD_CFG_APPEAR);
+    reg.dlg_proc = khm_cfg_appearance_proc;
+    LoadString(khm_hInstance, IDS_CFG_APPEAR_SHORT,
+               wshort, ARRAYLENGTH(wshort));
+    LoadString(khm_hInstance, IDS_CFG_APPEAR_LONG,
+               wlong, ARRAYLENGTH(wlong));
+
+    khui_cfg_register(NULL, &reg);
+
     reg.name = L"KhmIdentities";
     reg.dlg_template = MAKEINTRESOURCE(IDD_CFG_IDENTITIES);
     reg.dlg_proc = khm_cfg_identities_proc;
@@ -796,7 +960,7 @@ void khm_init_config(void) {
                wshort, ARRAYLENGTH(wshort));
     LoadString(khm_hInstance, IDS_CFG_IDS_TAB_LONG,
                wlong, ARRAYLENGTH(wlong));
-    reg.flags = KHUI_CNFLAG_SUBPANEL;
+    reg.flags = KHUI_CNFLAG_SUBPANEL | KHUI_CNFLAG_SYSTEM;
 
     khui_cfg_register(node, &reg);
 
@@ -807,11 +971,11 @@ void khm_init_config(void) {
                wshort, ARRAYLENGTH(wshort));
     LoadString(khm_hInstance, IDS_CFG_ID_TAB_LONG,
                wlong, ARRAYLENGTH(wlong));
-    reg.flags = KHUI_CNFLAG_PLURAL | KHUI_CNFLAG_SUBPANEL;
+    reg.flags = KHUI_CNFLAG_PLURAL | KHUI_CNFLAG_SUBPANEL | KHUI_CNFLAG_SYSTEM;
 
     khui_cfg_register(node, &reg);
 
-    reg.flags = 0;
+    reg.flags = KHUI_CNFLAG_SYSTEM;
     khui_cfg_release(node);
 
     reg.name = L"KhmNotifications";
