@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Massachusetts Institute of Technology
+ * Copyright (c) 2005 Massachusetts Institute of Technology
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -27,8 +27,11 @@
 #include<khuidefs.h>
 #include<kmm.h>
 #include<configui.h>
+#include<utils.h>
 #include<assert.h>
 
+#include<strsafe.h>
+
 khm_int32 cfgui_node_serial;
 LONG init_once = 0;
 CRITICAL_SECTION cs_cfgui;
@@ -39,7 +42,7 @@ static khui_config_node_i *
 cfgui_create_new_node(void) {
     khui_config_node_i * node;
 
-    node = malloc(sizeof(*node));
+    node = PMALLOC(sizeof(*node));
 #ifdef DEBUG
     assert(node);
 #endif
@@ -60,13 +63,13 @@ cfgui_free_node(khui_config_node_i * node) {
         return;
 
     if (node->reg.name)
-        free((void *) node->reg.name);
+        PFREE((void *) node->reg.name);
 
     if (node->reg.short_desc)
-        free((void *) node->reg.short_desc);
+        PFREE((void *) node->reg.short_desc);
 
     if (node->reg.long_desc)
-        free((void *) node->reg.long_desc);
+        PFREE((void *) node->reg.long_desc);
 
     node->magic = 0;
 
@@ -75,7 +78,7 @@ cfgui_free_node(khui_config_node_i * node) {
 
     ZeroMemory(node, sizeof(*node));
 
-    free(node);
+    PFREE(node);
 }
 
 
@@ -145,7 +148,7 @@ khui_cfg_register(khui_config_node vparent,
                               &cb_long_desc)) ||
         (vparent &&
          !cfgui_is_valid_node_handle(vparent)))
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     if (KHM_SUCCEEDED(khui_cfg_open(vparent,
                                   reg->name,
@@ -163,11 +166,11 @@ khui_cfg_register(khui_config_node vparent,
     node->reg = *reg;
     node->reg.flags &= KHUI_CNFLAGMASK_STATIC;
 
-    name = malloc(cb_name);
+    name = PMALLOC(cb_name);
     StringCbCopy(name, cb_name, reg->name);
-    short_desc = malloc(cb_short_desc);
+    short_desc = PMALLOC(cb_short_desc);
     StringCbCopy(short_desc, cb_short_desc, reg->short_desc);
-    long_desc = malloc(cb_long_desc);
+    long_desc = PMALLOC(cb_long_desc);
     StringCbCopy(long_desc, cb_long_desc, reg->long_desc);
 
     node->reg.name = name;
@@ -181,12 +184,28 @@ khui_cfg_register(khui_config_node vparent,
         parent = cfgui_node_i_from_handle(vparent);
     }
 
-    //node->owner = kmm_this_plugin();
+    /* plugin handles should not be obtained lightly.  For the moment,
+       the cleanup of nodes doesn't happen until module unload and
+       module unload doesn't happen until all the plugin and module
+       handles have been freed. */
+    /* node->owner = kmm_this_plugin(); */
 
     EnterCriticalSection(&cs_cfgui);
     TADDCHILD(parent, node);
+
+    if (hwnd_cfgui) {
+        SendMessage(hwnd_cfgui, KHUI_WM_CFG_NOTIFY,
+                    MAKEWPARAM(0, WMCFG_SYNC_NODE_LIST), 0);
+    }
+
     LeaveCriticalSection(&cs_cfgui);
 
+    /* when the root config list changes, we need to notify the UI.
+       this way, the Options menu can be kept in sync. */
+    if (parent == cfgui_root_config) {
+        kmq_post_message(KMSG_ACT, KMSG_ACT_SYNC_CFG, 0, 0);
+    }
+
     return KHM_ERROR_SUCCESS;
 }
 
@@ -204,7 +223,7 @@ khui_cfg_open(khui_config_node vparent,
          !cfgui_is_valid_node_handle(vparent)) ||
         FAILED(StringCbLength(name, KHUI_MAXCCH_NAME, &sz)) ||
         !result)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     EnterCriticalSection(&cs_cfgui);
     if (vparent)
@@ -238,11 +257,17 @@ KHMEXP khm_int32 KHMAPI
 khui_cfg_remove(khui_config_node vnode) {
     khui_config_node_i * node;
     if (!cfgui_is_valid_node_handle(vnode))
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     EnterCriticalSection(&cs_cfgui);
     node = cfgui_node_i_from_handle(vnode);
     node->flags |= KHUI_CN_FLAG_DELETED;
+
+    if (hwnd_cfgui) {
+        SendMessage(hwnd_cfgui, KHUI_WM_CFG_NOTIFY,
+                    MAKEWPARAM(0, WMCFG_SYNC_NODE_LIST), 0);
+    }
+
     LeaveCriticalSection(&cs_cfgui);
 
     return KHM_ERROR_SUCCESS;
@@ -251,7 +276,7 @@ khui_cfg_remove(khui_config_node vnode) {
 KHMEXP khm_int32 KHMAPI
 khui_cfg_hold(khui_config_node vnode) {
     if (!cfgui_is_valid_node_handle(vnode))
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     cfgui_hold_node(cfgui_node_i_from_handle(vnode));
 
@@ -261,13 +286,46 @@ khui_cfg_hold(khui_config_node vnode) {
 KHMEXP khm_int32 KHMAPI
 khui_cfg_release(khui_config_node vnode) {
     if (!cfgui_is_valid_node_handle(vnode))
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     cfgui_release_node(cfgui_node_i_from_handle(vnode));
 
     return KHM_ERROR_SUCCESS;
 }
 
+KHMEXP khm_int32 KHMAPI
+khui_cfg_get_parent(khui_config_node vnode,
+                    khui_config_node * result) {
+
+    khui_config_node_i * node;
+    khui_config_node_i * parent;
+
+    if(!cfgui_is_valid_node_handle(vnode) ||
+       !result)
+        return KHM_ERROR_INVALID_PARAM;
+
+    EnterCriticalSection(&cs_cfgui);
+    if (cfgui_is_valid_node_handle(vnode)) {
+        node = cfgui_node_i_from_handle(vnode);
+        parent = TPARENT(node);
+        if (parent == cfgui_root_config)
+            parent = NULL;
+    } else {
+        parent = NULL;
+    }
+    if (parent) {
+        cfgui_hold_node(parent);
+    }
+    LeaveCriticalSection(&cs_cfgui);
+
+    *result = parent;
+
+    if (parent)
+        return KHM_ERROR_SUCCESS;
+    else
+        return KHM_ERROR_NOT_FOUND;
+}
+
 KHMEXP khm_int32 KHMAPI
 khui_cfg_get_first_child(khui_config_node vparent,
                          khui_config_node * result) {
@@ -278,7 +336,7 @@ khui_cfg_get_first_child(khui_config_node vparent,
 
     if((vparent && !cfgui_is_valid_node_handle(vparent)) ||
        !result)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     EnterCriticalSection(&cs_cfgui);
     if (cfgui_is_valid_node_handle(vparent)) {
@@ -319,7 +377,7 @@ khui_cfg_get_first_subpanel(khui_config_node vparent,
 
     if((vparent && !cfgui_is_valid_node_handle(vparent)) ||
        !result)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     EnterCriticalSection(&cs_cfgui);
     if (cfgui_is_valid_node_handle(vparent)) {
@@ -360,7 +418,7 @@ khui_cfg_get_next(khui_config_node vnode,
 
     if (!cfgui_is_valid_node_handle(vnode) ||
         !result)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     EnterCriticalSection(&cs_cfgui);
     if (cfgui_is_valid_node_handle(vnode)) {
@@ -393,7 +451,7 @@ khui_cfg_get_next_release(khui_config_node * pvnode) {
 
     if (!pvnode || 
         !cfgui_is_valid_node_handle(*pvnode))
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     EnterCriticalSection(&cs_cfgui);
     if (cfgui_is_valid_node_handle(*pvnode)) {
@@ -429,7 +487,7 @@ khui_cfg_get_reg(khui_config_node vnode,
 
     if ((vnode && !cfgui_is_valid_node_handle(vnode)) ||
         !reg)
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     EnterCriticalSection(&cs_cfgui);
     if (cfgui_is_valid_node_handle(vnode)) {
@@ -447,7 +505,7 @@ khui_cfg_get_reg(khui_config_node vnode,
     if (node)
         return KHM_ERROR_SUCCESS;
     else
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 }
 
 KHMEXP HWND KHMAPI
@@ -552,6 +610,11 @@ khui_cfg_set_param(khui_config_node vnode, LPARAM param) {
     LeaveCriticalSection(&cs_cfgui);
 }
 
+static void
+clear_node_data(khui_config_node_i * node) {
+    node->n_data = 0;
+}
+
 static cfg_node_data *
 get_node_data(khui_config_node_i * node,
               void * key, 
@@ -575,7 +638,7 @@ get_node_data(khui_config_node_i * node,
 #ifdef DEBUG
         assert(node->nc_data >= node->n_data + 1);
 #endif
-        newdata = malloc(sizeof(*newdata) * node->nc_data);
+        newdata = PMALLOC(sizeof(*newdata) * node->nc_data);
 #ifdef DEBUG
         assert(newdata);
 #endif
@@ -583,12 +646,16 @@ get_node_data(khui_config_node_i * node,
 
         if (node->data && node->n_data > 0) {
             memcpy(newdata, node->data, node->n_data * sizeof(*newdata));
-            free(node->data);
+            PFREE(node->data);
         }
         node->data = newdata;
     }
 
     node->data[node->n_data].key = key;
+    node->data[node->n_data].hwnd = NULL;
+    node->data[node->n_data].param = 0;
+    node->data[node->n_data].flags = 0;
+
     node->n_data++;
 
     return &(node->data[node->n_data - 1]);
@@ -729,6 +796,8 @@ cfgui_clear_params(khui_config_node_i * node) {
     node->hwnd = NULL;
     node->param = 0;
     node->flags &= KHUI_CNFLAGMASK_STATIC;
+    clear_node_data(node);
+
     c = TFIRSTCHILD(node);
     while(c) {
         cfgui_clear_params(c);
@@ -767,8 +836,7 @@ khui_cfg_set_flags(khui_config_node vnode,
     mask &= KHUI_CNFLAGMASK_DYNAMIC;
 
     EnterCriticalSection(&cs_cfgui);
-    if (cfgui_is_valid_node_handle(vnode) &&
-        hwnd_cfgui != NULL) {
+    if (cfgui_is_valid_node_handle(vnode)) {
 
         node = cfgui_node_i_from_handle(vnode);
 
@@ -790,11 +858,12 @@ khui_cfg_set_flags(khui_config_node vnode,
 
 /* called with cs_cfgui held */
 static void
-recalc_node_flags(khui_config_node vnode) {
+recalc_node_flags(khui_config_node vnode, khm_boolean plural) {
     khui_config_node_i * node;
     khui_config_node_i * parent;
+    khui_config_node_i * subpanel;
+    cfg_node_data * data;
     khm_int32 flags;
-    khm_size i;
 
 #ifdef DEBUG
     assert(cfgui_is_valid_node_handle(vnode));
@@ -802,33 +871,43 @@ recalc_node_flags(khui_config_node vnode) {
 
     node = cfgui_node_i_from_handle(vnode);
 
-    parent = TPARENT(node);
+    if (plural)
+        parent = TPARENT(node);
+    else
+        parent = node;
 #ifdef DEBUG
     assert(parent);
 #endif
 
     flags = 0;
 
-    /* this code is wrong.  we need to go through all the subpanels in
-       the parent and pick the data record corresponding to this node
-       and then merge the flags from there. */
-    /* TODO: fix this */
-    for (i=0; i < parent->n_data; i++) {
-        if (parent->data[i].key == vnode)
-            flags |= parent->data[i].flags;
+    for(subpanel = TFIRSTCHILD(parent); subpanel;
+        subpanel = LNEXT(subpanel)) {
+        if (!(subpanel->reg.flags & KHUI_CNFLAG_SUBPANEL) ||
+            (plural && !(subpanel->reg.flags & KHUI_CNFLAG_PLURAL)) ||
+            (!plural && (subpanel->reg.flags & KHUI_CNFLAG_PLURAL)))
+            continue;
+
+        data = get_node_data(subpanel,
+                             vnode,
+                             FALSE);
+
+        if (data) {
+            flags |= data->flags;
+        }
     }
 
     flags &= KHUI_CNFLAGMASK_DYNAMIC;
 
-    if ((node->flags & flags) != flags) {
-        node->flags = flags |
-            (node->flags & ~KHUI_CNFLAGMASK_DYNAMIC);
+    if ((node->flags & KHUI_CNFLAGMASK_DYNAMIC) == flags)
+        return;
 
-        if (hwnd_cfgui)
-            PostMessage(hwnd_cfgui, KHUI_WM_CFG_NOTIFY,
-                        MAKEWPARAM((WORD) node->flags, WMCFG_UPDATE_STATE),
-                        (LPARAM) vnode);
-    }
+    node->flags = (node->flags & ~KHUI_CNFLAGMASK_DYNAMIC) | flags;
+
+    if (hwnd_cfgui)
+        PostMessage(hwnd_cfgui, KHUI_WM_CFG_NOTIFY,
+                    MAKEWPARAM((WORD) node->flags, WMCFG_UPDATE_STATE),
+                    (LPARAM) vnode);
 }
 
 KHMEXP void KHMAPI
@@ -862,7 +941,9 @@ khui_cfg_set_flags_inst(khui_config_init_data * d,
                 data->flags = new_flags;
 
                 if (d->ctx_node != d->ref_node)
-                    recalc_node_flags(d->ctx_node);
+                    recalc_node_flags(d->ctx_node, TRUE);
+                else
+                    recalc_node_flags(d->ctx_node, FALSE);
             }
         }
     }
@@ -879,8 +960,7 @@ khui_cfg_get_flags(khui_config_node vnode) {
         return 0;
 
     EnterCriticalSection(&cs_cfgui);
-    if (cfgui_is_valid_node_handle(vnode) &&
-        hwnd_cfgui != NULL) {
+    if (cfgui_is_valid_node_handle(vnode)) {
 
         node = cfgui_node_i_from_handle(vnode);
 
@@ -900,11 +980,10 @@ khui_cfg_get_name(khui_config_node vnode,
 
     if (!cb_buf ||
         !cfgui_is_valid_node_handle(vnode))
-        return KHM_ERROR_INVALID_PARM;
+        return KHM_ERROR_INVALID_PARAM;
 
     EnterCriticalSection(&cs_cfgui);
-    if (cfgui_is_valid_node_handle(vnode) &&
-        hwnd_cfgui != NULL) {
+    if (cfgui_is_valid_node_handle(vnode)) {
         khm_size cb;
 
         node = cfgui_node_i_from_handle(vnode);
@@ -919,7 +998,7 @@ khui_cfg_get_name(khui_config_node vnode,
             *cb_buf = cb;
         }
     } else {
-        rv = KHM_ERROR_INVALID_PARM;
+        rv = KHM_ERROR_INVALID_PARAM;
     }
     LeaveCriticalSection(&cs_cfgui);
 
@@ -936,7 +1015,7 @@ khui_cfg_init_dialog_data(HWND hwnd_dlg,
     khui_config_init_data * d;
 
     cb = sizeof(khui_config_init_data) + cb_extra;
-    d = malloc(cb);
+    d = PMALLOC(cb);
 #ifdef DEBUG
     assert(d);
 #endif
@@ -994,7 +1073,7 @@ khui_cfg_free_dialog_data(HWND hwnd_dlg) {
 #endif
 
     if (d) {
-        free(d);
+        PFREE(d);
     }
 
     return (d)?KHM_ERROR_SUCCESS: KHM_ERROR_NOT_FOUND;
